#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/21/2004
#
#------------------------------------------------------------------------------

""" Defines file editors for the wxPython user interface toolkit.
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import wx
from traits.api import (File, HasTraits, Button, Instance, Any, Callable, 
	Property, Directory, Bool)
from traitsui.wx.helper import TraitsUIPanel
from traitsui.api import (View, Item, CustomEditor, Handler, UIInfo)
from traitsui.file_dialog import open_file

from traitsui.wx.custom_editor import CustomEditor as CustomEditorKlass
from directory_dialog import open_directory

def mkeditor ( parent,editor,use_dir=False, *args ):
	""" Custom editor factory.  Must be instantiated on top of an
		InteractiveSubwindow instance with an implemented reconstruct().
	"""

	#IE the file trait is in the InteractiveSubwindow, not somewhere else

	editor.control = panel = TraitsUIPanel( parent, -1 )
	sizer        = wx.BoxSizer( wx.HORIZONTAL )

	if use_dir:
		editor.use_dir = True

	pad = 8

	bmp = wx.ArtProvider.GetBitmap ( wx.ART_FOLDER_OPEN, size= (15,15))
	button = wx.BitmapButton(panel, -1, bitmap=bmp)
	
	editor.text_control = text_control = wx.TextCtrl(panel, -1, '',
		style=wx.TE_PROCESS_ENTER)

	_do_update_obj = lambda ev:update_file_obj(ev,editor)
#	_do_update_obj = editor.update_file_obj

	wx.EVT_TEXT_ENTER( panel, text_control.GetId(), _do_update_obj)
	wx.EVT_KILL_FOCUS( text_control, _do_update_obj)

	sizer.Add( text_control, 1, wx.EXPAND | wx.ALIGN_CENTER )
	sizer.Add( button,  0, wx.RIGHT   | wx.ALIGN_CENTER, pad )

	wx.EVT_BUTTON( panel, button.GetId(),
		lambda ev:button_click(ev,editor) )
#		editor.button_click )
	panel.SetSizerAndFit( sizer )

	return panel

#methods referring to editor factory, which is not a real object
def button_click(event,editor):
	if editor.use_dir:
		file_selected=open_directory(entries=20)
	else:
		file_selected=open_file(entries=20)
	if file_selected:
		editor.value=file_selected
	#editor.object.reconstruct()
	editor.ui.handler.reconstruct()
	#editor.text_control.SetValue(editor.value)

def update_file_obj(event,editor):
	editor.value=editor.text_control.GetValue()

#custom editor on traitsui abstraction level
class CustomFileEditor(CustomEditor):
	'''abstraction layer for Custom File editor.  This editor must be
	instantiated within a view of an InteractiveSubwindow object with a
	correctly implemented reconstruct()'''
	#factory = Callable(mkeditor)
	factory = Property #Callable

	use_dir = Bool(False)

	#klass = Property

	def _get_klass(self):
		#tell the editor to use this instead of importing from traitsui.wx
		return CustomFileEditorKlass

	def _get_factory(self):
		if self.use_dir:
			return lambda p,e:mkeditor(p,e,True)
		else:
			return mkeditor

class CustomDirectoryEditor(CustomFileEditor):
	use_dir = True

#custom editor on level of toolkit (i.e., wx)
class CustomFileEditorKlass(CustomEditorKlass):
	text_control = Any # Instance( wx._control.TextCtrl )

	use_dir = Bool(False)

	def update_editor(self):
		self.text_control.SetValue(self.value)

	def button_click(self,event):
		#print "SKAGGGEY MUFFIN"
		file_selected=open_file()
		if file_selected:
			self.value=file_selected
		self.object.reconstruct()

	def update_file_obj(self,event):
		self.value=self.text_control.GetValue()

#test as main file
if __name__=='__main__':
	class Farkish(Handler):
		f=File('saukish')
		b=Button
		info=Instance(UIInfo)

		traits_view=View(
			Item('f',editor=CustomFileEditor()),
			Item('b'),
			height=500,width=500)

		def _b_fired(self):
			print self.f

		def init_info(self,info):
			self.info=info
		def reconstruct(self):
			self.info.ui.dispose()
			self.info.object.edit_traits()

	Farkish().configure_traits()



#	 (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is a modified version of UMCP, the UCLA Multimodal Connectivity
#	 Package.  UMCP is copyright (C) Jesse Brown 2012.  This file was modified
#	 06/2013.  For more information, see
#    http://www.ccn.ucla.edu/wiki/index.php/UCLA_Multimodal_Connectivity_Package
#
#    This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import struct
import numpy as np
import scipy.linalg as lin
import nibabel as nib
import random
from mayavi import mlab

def read_header(track_file):
	"""Read header values from a .trk (TrackVis) file and store them in a dict.
	This function replicates functionality from umcp, the UCLA multimodal 
	connectivity package, by Jesse Brown at UCLA"""
	header_dict={}
	f=open(track_file, 'rb') # added 'rb' for Windows reading
	contents = f.read()
	f.close()

	dims=(struct.unpack('3h',contents[6:12])[0])
	vox_size=(struct.unpack('3f',contents[12:24])[0])
	origin=(struct.unpack('3f',contents[24:36])[0])
	n_scalars=(struct.unpack('h',contents[36:38]))[0]
	n_properties=(struct.unpack('h',contents[238:240]))[0]
	vox_order=(struct.unpack('3c',contents[948:951])[0])
	paddings=(struct.unpack('3c',contents[952:955])[0])
	img_orient_patient=(struct.unpack('6f',contents[956:980])[0])
	inverts=(struct.unpack('3B',contents[982:985])[0])
	swaps=(struct.unpack('3B',contents[985:988])[0])
	num_fibers=(struct.unpack('i',contents[988:992])[0])
	header_dict["dims"]=dims
	header_dict["vox_size"]=vox_size
	header_dict["origin"]=origin
	header_dict["n_scalars"]=n_scalars
	header_dict["n_properties"]=n_properties
	header_dict["vox_order"]=vox_order
	header_dict["paddings"]=paddings
	header_dict["img_orient_patient"]=img_orient_patient
	header_dict["inverts"]=inverts
	header_dict["swaps"]=swaps
	header_dict["num_fibers"]=num_fibers

	#print np.array(contents[440:504],dtype=np.float32)
	#vox2ras=(struct.unpack('f',contents[440:504])[0])
	#print np.shape(vox2ras)
	#print vox2ras
	return header_dict

def read_tracks(track_file,display_nr=1500):
	"""Read tracks from a .trk file and store them in an array.  This function
	replicates functionality from the UCLA multimodal connectvity package,
	by Jesse Brown."""
	track_list=[]
	header=read_header(track_file)
	size=os.path.getsize(track_file)
	n_s=header['n_scalars']
	n_p=header['n_properties']
	f=open(track_file,'rb')
	contents=f.read(size)
	current=1000
	end=current+4
	nr_fibers=header['num_fibers']
	#calculate random indices
	selected_idxs=random.sample(xrange(nr_fibers),display_nr)

	i=0
	while end < size:		
		length = struct.unpack('i',contents[current:end])[0]
		current=end
		distance=length*(12+4*n_s)
		end = current+distance
		
		if end>size: #eof
			break
		if i not in selected_idxs:
			i+=1
			current=end+(4*n_p)
			end=current+4
			continue
		#else get the track
		floats=[]
		float_range = xrange(current,end,4)
		for float_start in float_range:
			float_end = float_start+4
			floats.append(struct.unpack('f',contents[float_start:float_end])[0])

		if n_p:
			properties_start=float_end
			property_start=properties_start
			#track_properties=[]	#properties are ignored
			for p in xrange(n_p):
				property_end=property_start+4
				#track_properties.append(struct.unpack(blah #properties ignored
				property_start=property_end+4
		
		floats=zip(*[iter(floats)] * 3)
		i+=1
		current=end+(4*n_p)
		end=current+4

		if len(floats)>0:
			track_list.append(np.array(floats))
	f.close()
	return track_list

def plot_naively(fname):
	from mayavi import mlab	

	f=read_tracks(fname)
	
	x=list()
	y=list()
	z=list()
	c=list()

	i=0
	for track in f:
		x.append(track[:,0])
		y.append(track[:,1])
		z.append(track[:,2])
		c.append([[j,j+1] for j in xrange(i,i+len(track)-1)])
		i+=len(track)

	x=np.hstack(x)
	y=np.hstack(y)
	z=np.hstack(z)
	c=np.vstack(c)

	s=np.zeros(len(x))

	src=mlab.pipeline.scalar_scatter(x,y,z,s,name='tractsrc')
	src.mlab_source.dataset.lines=c
	l=mlab.pipeline.stripper(src)
	sf=mlab.pipeline.surface(l,name='tractography')
	return sf

def rotate(x_like,y_like,angle=np.pi/9):
	#angle is in radians and clockwise

	xmap = lambda x,y:np.cos(angle)*x+np.sin(angle)*y
	ymap = lambda x,y:-np.sin(angle)*x+np.cos(angle)*y
	
	x=xmap(x_like,y_like)
	y=ymap(x_like,y_like)

	return x,y
	
def rescale(x,y,z,invert_y=True,**surf_properties_kwargs):
	import volume
	surf_xmin,surf_ymin,surf_zmin,surf_xmax,surf_ymax,surf_zmax=(
		volume.surf_properties(**surf_properties_kwargs))
	pad_nr=2
	surf_xmin+=pad_nr;	surf_xmax-=pad_nr
	surf_ymin+=pad_nr;	surf_ymax-=pad_nr
	surf_zmin+=pad_nr;	surf_zmax-=pad_nr

	xmax=np.max(x)
	xmin=np.min(x)
	xrng=xmax-xmin+1

	ymax=np.max(y)
	ymin=np.min(y)
	yrng=ymax-ymin+1

	zmax=np.max(z)
	zmin=np.min(z)
	zrng=zmax-zmin+1

	xmap=lambda x:(x-xmin)*(surf_xmax-surf_xmin+1)/xrng+surf_xmin
	if invert_y:
		ymap=lambda y:(y-ymin)*(surf_ymin-surf_ymax-1)/yrng+surf_ymax
		print surf_ymin-surf_ymax-1
		print yrng
	else:
		ymap=lambda y:(y-ymin)*(surf_ymax-surf_ymin+1)/yrng+surf_ymin
	zmap=lambda z:(z-zmin)*(surf_zmax-surf_zmin+1)/zrng+surf_zmin

	retx=xmap(x)
	rety=ymap(y)
	retz=zmap(z)
	
	return retx,rety,retz

def affine(affine_trans_file):
	a=np.loadtxt(affine_trans_file)
	return a

def reverse_affine(a):
	#this is equivalent to simply returning lin.inv(a) but has slightly better
	#performance, albeit on the order of .2 ms better
	r=a[0:3,0:3]
	t=a[0:3,3]

	ri=lin.inv(r)
	api=np.zeros((4,4))
	api[0:3,0:3]=ri
	api[0:3,3]=np.dot(ri,t*-1)
	api[3,3]=1
	return api

def apply_affine(a,s):
	tx,ty,tz=s.mlab_source.x,s.mlab_source.y,s.mlab_source.z

	ones=np.ones(len(tx))
	dat=np.array(zip(tx,ty,tz,ones))

	for i,d in enumerate(dat):
		dat[i]=np.dot(a,d)

	s.mlab_source.x,s.mlab_source.y,s.mlab_source.z,_ = dat.T

def plot_fancily(fname,ang=-np.pi/18):
	s=plot_naively(fname)

	s.actor.property.opacity=.1
	s.module_manager.scalar_lut_manager.lut_mode='Purples'
	s.module_manager.scalar_lut_manager.reverse_lut=True
	mlab.view(focalpoint='auto')
	return s

def apply_cmp_affines(s,b0vol_fname,trk_fname,fs_subj_name,
		fs_subjs_dir,fsenvsrc=None):
	'''applies B0-TKR transform and then TKR-RAS transform
		
	Include the fsenvsrc parameter if nmrenv has not already been set'''

	if fsenvsrc:
		import cvu_utils
		cvu_utils.tcsh_env_interpreter(fsenvsrc)

	a1=fetch_b0_to_tkr(b0vol_fname,trk_fname)
	a2=fetch_ras_to_tkr(b0vol_fname,fs_subj_name,fs_subjs_dir)

	a2i=reverse_affine(a2)

	co=np.dot(a1,a2) #compose these transformations

	apply_affine(co,s)
	mlab.view(focalpoint='auto')

def fetch_b0_to_tkr(b0vol_fname,trk_fname,fsenvsrc=None,out_fname=None):
	'''automated function to generate transformation matrix from B0 volume
	to volume RAS coordinates.  first of two automated transform generator
	functions and considerably simpler than the next one.  if out_fname is not 
	supplied, returns a numpy matrix containing the affine transform.

	Include the fsenvsrc parameter if nmrenv has not already been set.'''

	if fsenvsrc:
		import cvu_utils
		cvu_utils.tcsh_env_interpreter(fsenvsrc)

	v=1.0/read_header(trk_fname)['vox_size']

	import subprocess
	cmd = 'mri_info %s --vox2ras-tkr' % (b0vol_fname)

	proc=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
	crs_to_tkr_arr=np.loadtxt(proc.stdout)

	b0_to_crs_arr=np.diag((v,v,v,1))

	ret = np.dot(b0_to_crs_arr,crs_to_tkr_arr)

	if out_fname:
		np.savetxt(out_fname,ret)
	else:
		return ret

def fetch_ras_to_tkr(b0vol_fname,fs_subj_name,fs_subjects_dir,fsenvsrc=None
		,out_fname=None):
	'''automated function to generate transformation from surface RAS to volume
	RAS coordinates.  bbregister is used to do this.  this function takes a few 	minutes. if out_fname is not supplied, returns a numpy matrix containing the
	affine transform.

	Include the fsenvsrc if nmrenv has not already been set.''' 

	import subprocess; import tempfile; import glob

	if fsenvsrc:
		import cvu_utils
		cvu_utils.tcsh_env_interpreter(fsenvsrc)
	os.environ['SUBJECTS_DIR']=fs_subjects_dir

	tmpfd,tmpfname=tempfile.mkstemp()
	cmd = ('bbregister --s %s --mov %s --init-fsl --reg \'%s\' --bold '
		'--tol1d 1e-3' % (fs_subj_name, b0vol_fname, tmpfname+".reg"))

	#print os.environ['SUBJECTS_DIR']
	#print cmd

	with open(os.devnull,'wb') as devnull:
		proc=subprocess.Popen(cmd,shell=True,stdout=devnull)

	if proc.wait() != 0:
		for i in glob.glob('tmpfname*'):
			os.unlink(i)
		raise cvu_utils.CVUError("bbregister crashed with error %i" %
			proc.returncode)

	with open(tmpfname+'.reg','rwb') as tmpf:
		for i in xrange(4):
			tmpf.readline() #ignore first 4 lines
		with os.tmpfile() as tmpf2:
			for j in xrange(4):
				tmpf2.write(tmpf.readline()+'\n')
			tmpf2.seek(0)
			ret = np.loadtxt(tmpf2) 
		
	for i in glob.glob('tmpfname*'):
		os.unlink(i)
	#TODO also unlink all the other things this creates in /tmp

	if out_fname:
		np.savetxt(out_fname,ret)
	else:
		return ret



#    (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from traits.api import (HasTraits,Bool,Event,File,Int,Str,Directory,Function,
	Enum,List,Button,Range,Instance,Float,Trait,CFloat)
from traitsui.api import (Handler,View,Item,OKCancelButtons,OKButton,Spring,
	Group,ListStrEditor,CheckListEditor,HSplit,FileEditor,VSplit,Action,HGroup,
	TextEditor,ImageEnumEditor,UIInfo,Label,VGroup,ListEditor)
from traitsui.file_dialog import open_file
import os; import cvu_utils as util;
from color_map import CustomColormap
from custom_file_editor import CustomFileEditor, CustomDirectoryEditor

from traits.api import on_trait_change

class InteractiveSubwindow(Handler):
	finished=Bool(False)
	notify=Event

	info=Instance(UIInfo)

	def init_info(self,info):
		self.info=info
	def closed(self,info,is_ok):
		info.object.finished=is_ok
		info.object.notify=True
	def reconstruct(self,info=None):
		if info is None:
			info=self.info
		info.ui.dispose()
		info.object.edit_traits()

def append_proper_buttons(buttonlist):
	#get around list mutability
	buttonlist.extend(OKCancelButtons)
	return buttonlist

class OptionsWindow(InteractiveSubwindow):
	surface_visibility = Range(0.0,1.0,.15)
	circ_size = Range(7,20,10,mode='spinner')
	conns_colorbar=Bool(False)
	scalar_colorbar=Bool(False)
	pthresh = Range(0.0,1.0,.95)
	nthresh = Float
	thresh_type = Enum('prop','num')
	prune_modules = Bool(True)
	show_floating_text = Bool(True)
	module_view_style = Enum('intramodular','intermodular','both')
	render_style=Enum('glass','cracked_glass','contours','wireframe','speckled')
	conns_disclaimer=Str
	interhemi_conns_on = Bool(True)
	lh_conns_on = Bool(True)
	rh_conns_on = Bool(True)
	lh_nodes_on = Bool(True)
	rh_nodes_on = Bool(True)
	lh_surfs_on = Bool(True)
	rh_surfs_on = Bool(True)
	conns_width = Float(2.)
	conns_colors_on = Bool(True)

	intermediate_graphopts_list=List(Str)
	initial_graphopts_list=List(Str)
	modules_list=List(Str)
	traits_view=View(
		VGroup(
			HSplit(
				Item(name='circ_size'),
				Item(name='conns_width',label='conn linewidth'),
				Item(name='show_floating_text',label='floating 3D text on'),
			),
			HSplit(
				Item(name='pthresh'),
				Item(name='nthresh'),
				Item(name='thresh_type'),
			),
			HSplit(
				Item(name='render_style',label='surface style'),
				Item(name='surface_visibility',label='surface opacity'),
			),
			HSplit(
				Item(name='module_view_style',label='module connection style'),
				#Item(name='prune_modules',label='prune singleton modules'),
			),
			HSplit(
				Item(name='interhemi_conns_on',
					label='interhemispheric conns on'),
				Item(name='lh_conns_on',label='LH conns on'),
				Item(name='rh_conns_on',label='RH conns on'),
			),
			HSplit(
				Item(name='lh_nodes_on',label='LH nodes on'),
				Item(name='rh_nodes_on',label='RH nodes on'),
			),
			HSplit(
				Item(name='lh_surfs_on',label='LH surfaces on'),
				Item(name='rh_surfs_on',label='RH surfaces on'),
			),
			HSplit(
				Item(name='conns_colorbar',label='show conns colorbar'),
				Item(name='scalar_colorbar',label='show scalar colorbar'),
				Item(name='conns_colors_on'),
			),
			HSplit(
				Item(name='conns_disclaimer',style='readonly',height=10,
					width=550,show_label=False)
			),
			label='Display options',show_labels=False
		),
		VGroup(
			Item(name='intermediate_graphopts_list',editor=CheckListEditor(
				name='initial_graphopts_list'),show_label=False,
				style='custom'),
			label='Graph statistics',show_labels=False
		),
		kind='live',buttons=OKCancelButtons,
		title='Select your desired destiny',
	)

	def _initial_graphopts_list_default(self):
		return ['global efficiency', 'local efficiency', 'average strength',
			'clustering coefficient', 'eigenvector centrality', 'modularity', 
			'participation coefficient', 'within-module degree']
	def _intermediate_graphopts_list_default(self):
		return ['global efficiency', 'clustering coefficient']
	def _conns_disclaimer_default(self):
		return ("Note changing conn visibility is not applied immediately as "
			"it can be costly. To force application, click 'Reset Display'")

class GraphTheoryWindow(InteractiveSubwindow):
	from graph import StatisticsDisplay

	graph_stats=List(StatisticsDisplay)
	current_stat=Instance(StatisticsDisplay)
	scalar_savename=Str

	RecalculateButton=Action(name='Recalculate',action='do_recalculate')
	RecalculateEvent=Event

	SaveToScalarButton=Action(name='Save to scalar',action='do_sv_scalar')
	SaveToScalarEvent=Event
	
	traits_view=View(
		VGroup(
			HGroup(
			Item('graph_stats',style='custom',show_label=False,
				editor=ListEditor(use_notebook=True,page_name='.name',
				selected='current_stat')),
			),HGroup(
			Item('scalar_savename',label='Scalar name',
				height=25,width=180),
			),
		),
		height=400,width=350,
		title='Mid or feed',kind='live',
		buttons=[RecalculateButton,SaveToScalarButton,OKButton,])

	def _current_stat_changed(self):
		self.scalar_savename=self.current_stat.name

	#handler methods
	def do_sv_scalar(self,info):
		self.SaveToScalarEvent=True

	def do_recalculate(self,info):
		self.RecalculateEvent=True

class CustomColormapWindow(InteractiveSubwindow):
	#first argument is class, second is default value.
	default_map=Instance(CustomColormap,CustomColormap('default'))
	scalar_map=Instance(CustomColormap,CustomColormap('scalar'))
	activation_map=Instance(CustomColormap,CustomColormap('activation'))
	connmat_map=Instance(CustomColormap,CustomColormap('connmat'))

	EditCmapButton = Action(name='Colormap customizer',action='do_edit_cmap')
	ResetDefaultsButton = Action(name='Reset defaults',action='do_defaults')

	def cmap_group_view(m):
		return Group(Item(name='label',style='readonly',object=m),
					Item(name='cmap',object=m,
						editor=ImageEnumEditor(path=CustomColormap.imgs_path,
							values=CustomColormap.lut_list,cols=7)),
					Item(name='fname',editor=CustomFileEditor(),object=m,
						enabled_when='%s.cmap==\'file\''%m),
					HGroup(
						Item(name='reverse',object=m)
					),
					Item(name='threshold',object=m,
						enabled_when='%s.cmap==\'custom_heat\''%m),
					show_labels=False)

	traits_view=View(
		HGroup(
			cmap_group_view('default_map'),
			cmap_group_view('scalar_map'),
			cmap_group_view('activation_map'),
			cmap_group_view('connmat_map')
		),
		kind='live',
		buttons=append_proper_buttons([EditCmapButton,ResetDefaultsButton]),
		title='If it were up to me it would all be monochrome',
	)

	#handler methods
	def do_edit_cmap(self,info):
		from tvtk import util as tvtk_util; import sys,subprocess
		script=os.path.join(os.path.dirname(tvtk_util.__file__),
			'wx_gradient_editor.py')
		subprocess.Popen([sys.executable, script])

	def do_defaults(self,info):
		for map in [self.default_map,self.scalar_map,self.activation_map,
				self.connmat_map]:
			map.reset_traits(['cmap','fname','reverse','threshold'])

	def edit_traits(self):
		super(CustomColormapWindow,self).edit_traits(context=
			{'default_map':self.default_map,
			 'scalar_map':self.scalar_map,
			 'activation_map':self.activation_map,
			 'connmat_map':self.connmat_map,
			 'object':self})

class RequireWindow(InteractiveSubwindow):
	require_ls=List(Str)
	please_note=Str('Enter the ROIs you would like to force to display on the '
		'circle plot.  You must spell them precisely, e.g. "lh_frontalpole"')
	traits_view=View(
		Item(name='please_note',style='readonly',height=35,width=250),
		Item(name='require_ls',editor=ListStrEditor(auto_add=True,
			editable=True),label='List ROIs here'),
		buttons=OKCancelButtons,title='Mango curry')

class AdjmatChooserWindow(InteractiveSubwindow):
	Please_note=Str("All but first field are optional.  Specify adjmat order "
		"if the desired display order differs from the existing matrix order."
		"  Specify unwanted channels as 'delete' in the label order.  Data "
		"field name applies to the data field for .mat matrices only.")
	adjmat=File
	open_adjmat=Button('Browse')
	open_adjmat_order=Button('Browse')
	#adjmat_order=Trait(None,None,File)
	adjmat_order=File
	max_edges=Int
	field_name=Str('adj_matrices')
	ignore_deletes=Bool
	require_window=Instance(InteractiveSubwindow,())
	RequireButton=Action(name='force display of ROIs',action='do_rw_show')
	traits_view=View(
		Item(name='Please_note',style='readonly',height=140,width=250),
		#HSplit(
		#	Item(name='adjmat',style='text'),
		#	Item(name='open_adjmat',show_label=False),
		#),
		Item(name='adjmat',
			editor=CustomFileEditor()),
		Item(name='adjmat_order',label='Label Order',
			editor=CustomFileEditor()),
		#HSplit(
		#	Item(name='adjmat_order',style='text'),
		#	Item(name='open_adjmat_order',show_label=False),
		#),
		Item(name='max_edges',label='Max Edges'),
		Item(name='field_name',label='Data Field Name'),
		Item(name='ignore_deletes',label='Ignore deletes'),
		kind='live',buttons=append_proper_buttons([RequireButton]),
		title='Report all man-eating vultures to security',)

	#private methods
	def _open_adjmat_fired(self):
		self.adjmat=open_file()
		self.edit_traits()
	def _open_adjmat_order_fired(self):
		self.adjmat_order=open_file()
		self.edit_traits()
	def _require_window_default(self):
		return RequireWindow()

	#handler methods
	def do_rw_show(self,info):
		info.object.require_window.edit_traits()

class ParcellationChooserWindow(InteractiveSubwindow):
	Please_note=Str('fsaverage5 is fine unless individual morphology '
		'is of interest.  Visualizing tractography requires individual '
		'morphology.')
	SUBJECTS_DIR=Directory('./')
	SUBJECT=Str('fsavg5')
	labelnames_f=File
	open_labelnames_f=Button('Browse')
	parcellation_name=Str
	DisposeEvent=Action(action='do_dispose')
	traits_view=View(
		Group(
			Item(name='Please_note',style='readonly',height=85,width=250),
			Item(name='SUBJECT'),
			Item(name='SUBJECTS_DIR',editor=CustomDirectoryEditor()),
			Item(name='parcellation_name',label='Parcellation'),
			Item(name='labelnames_f',label='Label Display Order',
				editor=CustomFileEditor()),
			#HSplit(
			#	Item(name='labelnames_f',label='Label Display Order',
			#		style='text',springy=True),
			#	Item(name='open_labelnames_f',show_label=False)
			#),
		), kind='live',buttons=OKCancelButtons,
			title="This should not be particularly convenient",)

	def _open_labelnames_f_fired(self):
		self.labelnames_f=open_file()
		self.do_dispose(self.info)
		self.edit_traits()

	def _reconstruct(self):
		self.do_dispose(self.info)
		self.edit_traits()

class TractographyChooserWindow(InteractiveSubwindow):
	Please_note=Str('Tractography will be misaligned with the surface unless '
		'the surface corresponds to the subject\'s individual morphology.\n'
		'cvu will source the freesurfer setup script.  You can omit this if '
		'the environment has already been set prior to running cvu.  All four '
		'of the other fields are required.')
	track_file=File
	b0_volume=File
	SUBJECTS_DIR=Directory
	SUBJECT=Str
	fs_setup=File('/usr/local/freesurfer/nmr-stable53-env')
	traits_view=View(
		Group(
			Item(name='Please_note',style='readonly',height=125,width=325),
			Item(name='track_file',editor=CustomFileEditor()),
			Item(name='b0_volume',editor=CustomFileEditor()),
			Item(name='SUBJECTS_DIR',editor=CustomDirectoryEditor()),
			Item(name='SUBJECT'),
			Item(name='fs_setup',editor=CustomFileEditor()),
		), kind='live',buttons=OKCancelButtons,
			title='Just FYI subject 39108 has an abnormal HRF')

class LoadGeneralMatrixWindow(InteractiveSubwindow):
	Please_note=Str('Same rules for adjmat ordering files apply')
	mat=File
	open_mat=Button('Browse')
	mat_order=File
	field_name=Str
	ignore_deletes=Bool
	whichkind=Enum('modules','scalars')
	dataset_nr=Int(1)
	dataset_name=Str('dataset1')
	traits_view=View(
		Item(name='Please_note',style='readonly',height=50,width=250),
		Item(name='mat',label='Filename',editor=CustomFileEditor()),
		#HSplit(
		#	Item(name='mat',label='Filename',style='text',springy=True),
		#	Item(name='open_mat',show_label=False),
		#),
		#Item(name='mat',label='Filename',editor=FileEditor(entries=10),style='simple'),
		Item(name='mat_order',label='Ordering file',editor=CustomFileEditor()),
		Item(name='field_name',label='Field (.mat files only)'),
		Item(name='ignore_deletes',label='Ignore deletes'),
		Item(name='dataset_name',label='Name this dataset'),
		kind='live',buttons=OKCancelButtons,
		title='Behold the awesome power of zombies')

	def _open_mat_fired(self):
		#self.mat=open_file()
		res=util.file_chooser(initialdir=os.path.dirname(self.mat),
			title='Roentgenium is very useful')
		if len(res)>0:
			self.mat=res

	def dataset_plusplus(self):
		self.dataset_nr+=1
		self.dataset_name='dataset%i' % self.dataset_nr

class ConfigureScalarsWindow(InteractiveSubwindow):
	#idea:
	#node_scalars needs to be DICTIONARY.
	#add field to dictionary upon load scalars.  can replace.
	#three ListStr editors, one for each display method, across the field names
	#the selections can be the same for multiple scalars
	scalar_sets=List(Str)
	nod_col=Str
	nc_str=Str('Node color')
	srf_col=Str
	sc_str=Str('Surface color')
	nod_siz=Str
	ns_str=Str('Node size')
	circle=Str
	circ_str=Str('Circle plot')
	conmat=Str
	cmat_str=Str('Connection Matrix')
	traits_view=View(
		HGroup(
			Group(
				Item('nc_str',style='readonly'),
				Item('scalar_sets',editor=ListStrEditor(selected='nod_col')),
				show_labels=False
			),
			Group(
				Item('sc_str',style='readonly'),
				Item('scalar_sets',editor=ListStrEditor(selected='srf_col')),
				show_labels=False
			),
			Group(
				Item('ns_str',style='readonly'),
				Item('scalar_sets',editor=ListStrEditor(selected='nod_siz')),
				show_labels=False
			),
			Group(
				Item('circ_str',style='readonly'),
				Item('scalar_sets',editor=ListStrEditor(selected='circle')),
				show_labels=False
			),
			Group(
				Item('cmat_str',style='readonly'),
				Item('scalar_sets',editor=ListStrEditor(selected='conmat')),
				show_labels=False
			),
		),
		height=200,width=800,buttons=OKCancelButtons,
		title='Your data is probably just spurious artifacts anyway',
	)
	
class NodeChooserWindow(InteractiveSubwindow):
	node_list=List(Str)
	cur_node=Int(-1)
	traits_view=View(
		Item(name='node_list',editor=
			ListStrEditor(selected_index='cur_node'),show_label=False),
		kind='live',height=350,width=350,buttons=OKCancelButtons,
		resizable=True,title='Do you know the muffin man?')

class ModuleChooserWindow(InteractiveSubwindow):
	module_list=List(Str)
	cur_mod=Int(-1)
	traits_view=View(
		Item(name='module_list',
			editor=ListStrEditor(editable=True,selected_index='cur_mod'),
			show_label=False),
		kind='live',height=350,width=350,buttons=OKCancelButtons,
		resizable=True,title='Roll d12 for dexterity check')

class ModuleCustomizerWindow(InteractiveSubwindow):
	initial_node_list=List(Str)
	intermediate_node_list=List(Str)
	return_module=List(Int)
	ClearButton=Action(name='Clear Selections',action='do_clear')
	traits_view=View(
		Item(name='intermediate_node_list',editor=CheckListEditor(
			name='initial_node_list',cols=2),show_label=False,style='custom'),
		kind='live',height=400,width=500,
		buttons=append_proper_buttons([ClearButton]),
		resizable=True,scrollable=True,title='Mustard/Revolver/Conservatory')

	#index_convert may return a ValueError, it should be
	#contained in try/except from higher up.
	def index_convert(self):
		self.return_module=[self.initial_node_list.index(i)
			for i in self.intermediate_node_list]

	#handler methods
	def do_clear(self,info):
		info.object.intermediate_node_list=[]

class SaveSnapshotWindow(InteractiveSubwindow):
	savefile=Str(os.environ['HOME']+'/')
	dpi=Int(300)
	whichplot=Enum('3D brain','connection matrix','circle plot')
	traits_view=View(Group(
		Item(name='savefile'),
		Item(name='whichplot',label='view'),
		Item(name='dpi',label='dots per inch'),
	), kind='live',buttons=OKCancelButtons,
		title="Help I'm a bug",height=250,width=250)

class MakeMovieWindow(InteractiveSubwindow):
	savefile=Str(os.environ['HOME']+'/')
	framerate=Int(20)
	bitrate=Int(4000) 
	samplerate=Int(8)
	#use x11grab exclusively.  remove snapshots altogether eventually
	type=Enum('x11grab','snapshots')
	anim_style=Bool(True)
	animrate=Int(8)
	traits_view=View(Group(
		Item(name='savefile'),
		Item(name='framerate',label='framerate'),
		Item(name='bitrate',label='bitrate (kb/s)'),
		Item(name='anim_style',label='automatically rotate'),
		Item(name='samplerate',label='animation speed'),
	), kind='live',buttons=OKCancelButtons,
		title="Make me a sandwich",height=250,width=450)

class ReallyOverwriteFileWindow(InteractiveSubwindow):
	Please_note=Str('That file exists.  Really overwrite?')
	save_continuation=Function # Continuation passing style
	traits_view=View(Spring(),
		Item(name='Please_note',style='readonly',height=25,width=250,
			show_label=False),
		Spring(),
		kind='live',buttons=OKCancelButtons,
		title='Your doom awaits you')

class ErrorDialogWindow(HasTraits):
	error=Str
	traits_view=View(Item(name='error',style='readonly',height=75,width=300),
		buttons=[OKButton],kind='nonmodal',
		title='Evil mutant zebras did this',)

class WarningDialogWindow(HasTraits):
	warning=Str
	traits_view=View(Item(name='warning',style='readonly',height=75,width=300),
		buttons=[OKButton],kind='nonmodal',
		title='Evil mutant zebras did this',)

class AboutWindow(HasTraits):
	message=Str('cvu version 0.2\n'
		'cvu is copyright (C) Roan LaPlante 2013\n\n'
		'cvu strictly observes the tenets of fundamentalist Theravada Mahasi\n'
		'style Buddhism.  Any use of cvu in violation of the tenets of\n'
		'fundamentalist Theravada Mahasi style Buddhism or in violation of\n'
		'the theosophical or teleological principles as described\n'
		'in the Vishuddhimagga Sutta is strictly prohibited and punishable by\n'
		'extensive Mahayana style practice.  By being or not being mindful of\n'
		'the immediate present moment sensations involved in the use of cvu,\n'
		'you confer your acceptance of these terms and conditions.\n\n'

		'cvu is distributed without warranty unless otherwise prohibited by\n'
		'law.  cvu is licensed under the GNU GPL v3.  a license is contained\n'
		'with the distribution of this program.  you may modify and convey\n'
		'this program to others in accordance with the terms of the GNU GPLv3\n'		'or optionally any subsequent version of the GNU GPL.'
	)
	traits_view=View(Item(name='message',style='readonly',show_label=False),
		buttons=[OKButton],kind='nonmodal',height=300,width=300,
		title='Greetings, corporeal being',)



#    (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from chaco.api import PlotAxis,LinearMapper
import numpy as np
from traits.api import RGBColor,List,Any,Enum,HasTraits,Instance,Int

def rcol():
	return tuple(np.random.random(3)).__add__((1,))

class ColorfulAxis(PlotAxis):
	colors=List(Any)
	direction=Enum('x','y',None)

	def __init__(self,component,color_list,direction,**traits):
		super(ColorfulAxis,self).__init__(component=component,**traits)
	
		self.colors=color_list
		self.ensure_labels_bounded=True
		self.axis_line_visible=False
		self.tick_interval=1
		self.tick_weight=2

		self.direction=direction
		if self.direction=='x':
			self.orientation='bottom'
		elif self.direction=='y':
			self.orientation='left'
	
	def _draw_ticks(self,gc):
		if not self.tick_visible:
			return
		gc.set_line_width(self.tick_weight)
		gc.set_antialias(False)
		#tick_in_vector = self._inside_vector*self.tick_in
		tick_out_vector = self._inside_vector*self.tick_out*2
		#print self._tick_positions
		
		if self.direction=='x':
			self.mapper=self.component.x_mapper
		elif self.direction=='y':
			self.mapper=self.component.y_mapper

		min_scr=self.mapper.range.low
		max_scr=self.mapper.range.high

		#deal with the edge cases
		if min_scr-int(min_scr)==0:
			#the tick at r is preserved and i'll put at r+.5
			min_r=int(min_scr)
		elif min_scr-int(min_scr)<=.5:
			#i need a tick at r+.5 but i don't have one, take the one from r-1
			min_r=int(np.floor(min_scr))
		elif min_scr-int(min_scr)>.5:
			#don't need a tick at r+.5, don't have a tick at r, start at r+1
			min_r=int(np.ceil(min_scr))
		else:
			raise IndexError('Internal error in ColorfulAxis min')

		if max_scr-int(max_scr)<.5:
			#i have a superfluous tick at r, get rid of it
			max_r=int(np.floor(max_scr))
		elif max_scr-int(max_scr)>=.5:
			#the tick at r will be displayed at r+.5, stop at r+1
			max_r=int(np.ceil(max_scr))
		else:
			raise IndexError('Internal error in ColorfulAxis max')

		moddiv=int(np.ceil(max((max_r-min_r)//75,1)))
		inds=xrange(min_r,max_r,moddiv)
		nr_ticks=len(inds)

		# add .5 to each tick to place them in the center of the grid
		ticks_axis=self.mapper.map_screen(np.array(inds)+.5)
		ticks_static=np.tile(48,(nr_ticks,))

		if self.direction=='x':
			ticks=np.vstack((ticks_axis,ticks_static)).T
		elif self.direction=='y':
			ticks=np.vstack((ticks_static,ticks_axis)).T

		for tick_pos,i in zip(ticks,inds):
			if i<0 or i>=len(self.colors):
				#gc.set_stroke_color((0,0,0))
				continue
			else:
				gc.set_stroke_color(self.colors[i])
			gc.begin_path()
			gc.move_to(*(tick_pos))
			gc.line_to(*(tick_pos - tick_out_vector))
			gc.stroke_path()
		return

	def _draw_labels(self,gc):
		pass



import os
from traits.api import (Directory, Str)
from traitsui.api import (Item, HGroup, VGroup, View, HistoryEditor,
	DirectoryEditor)
from traitsui.file_dialog import OpenFileDialog

#reimplementation of OpenFileDialog for directory selection

class OpenDirectoryDialog(OpenFileDialog):
	file_name = Directory

	id = Str('OpenDirectoryDialog')

	def _get_is_valid_file(self):
		if self.is_save_file:
			return (os.path.isdir(self.file_name) or (os.path.exists(
				self.file_name)))

		return os.path.isdir(self.file_name) or os.path.isfile(self.file_name)

	def open_file_view(self):
		item=Item( 'file_name',
					id      = 'file_tree',
					style   = 'custom',
					show_label = False,
					width   = 0.5,
					editor = DirectoryEditor( filter = self.filter,
											allow_dir = True,
											reload_name = 'reload',
											dclick_name = 'dclick',))
		width=height=0.20

		if len(self.extensions) > 0:
			raise Exception('extensions are not supported')


		return View (
			VGroup(
				VGroup( item ),
				HGroup(
					Item( 'create',
						  id           = 'create',
						  show_label   = False,
						  style        = 'custom',
						  defined_when = 'is_save_file',
						  enabled_when = 'can_create_dir',
						  tooltip      = 'Create a new directory'
					),
					Item( 'file_name',
						  id      = 'history',
						  editor  = HistoryEditor( entries  = self.entries,
												   auto_set = True ),
						  springy = True
					),
					Item( 'ok',
						  id           = 'ok',
						  show_label   = False,
						  enabled_when = 'is_valid_file'
					),
					Item( 'cancel',
						  show_label = False
					)
				)
			),
			title        = self.title,
			id           = self.id,
			kind         = 'livemodal',
			width        = width,
			height       = height,
			close_result = False,
			resizable    = True
		)

def open_directory(**traits):
	fd=OpenDirectoryDialog(**traits)
	if fd.edit_traits( view='open_file_view' ).result:
		return fd.file_name
	else:
		return ''




# (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu

import bct
import numpy as np
import scipy.io as sio
from collections import OrderedDict
from traits.api import HasTraits,Str,Any,List

class StatisticsDisplay(HasTraits):
	from traitsui.api import View,Item,TabularEditor
	#from traitsui.ui_editors.array_view_editor import ArrayViewEditor
	from traitsui.tabular_adapter import TabularAdapter
	name=Str
	stat=Any	#np.ndarray
	chart=Any	#np.ndarray
	label_names=List(Str)

	def __init__(self,name,stat,labels,**kwargs):
		super(HasTraits,self).__init__(**kwargs)
		self.name=name
		if np.size(stat)==1:
			self.stat=stat
			self.chart=np.array((('','%.3f'%stat,),))
		elif np.size(stat)!=len(labels):
			print np.size(stat),len(labels)
			raise ValueError('Size of graph statistic inconsistent')
		else:
			nr_labels=len(labels)
			self.stat=stat.reshape((nr_labels,1))
			self.chart=np.append(
				np.reshape(labels,(nr_labels,1)),
				np.reshape(map(lambda nr:'%.3f'%nr,stat),(nr_labels,1)),
				axis=1)
		#self.stat=np.reshape(stat,(np.size(stat),1))

	traits_view=View(
		#Item('stat',editor=ArrayViewEditor(show_index=False,format='%.4f'),
		#	height=350,width=225,show_label=False),
		Item('chart',editor=TabularEditor(
			adapter=TabularAdapter(columns=['','']),
			editable=False,show_titles=True),
			height=300,width=225,show_label=False),
	)

def do_summary(adj,mods,opts):
	stats=OrderedDict()
	for opt in opts:
		#throw an error if modularity calculations were requested but no
		#community structure exists
		if opt in ('modularity','participation coefficient','within-module '
				'degree') and mods is None:
			import cvu_utils as util
			raise util.CVUError('Need Modules')
	for opt in opts:
		stats.update({opt:do_opt(adj,mods,opt)})
	return stats

def do_opt(adj,mods,option):
	if option=='global efficiency':
		return bct.efficiency_wei(adj)
	elif option=='local efficiency':
		return bct.efficiency_wei(adj,local=True)
	elif option=='average strength':
		return bct.strengths_und(adj)
	elif option=='clustering coefficient':
		return bct.clustering_coef_wu(adj)
	elif option=='eigenvector centrality':
		return bct.eigenvector_centrality_und(adj)

	elif option=='modularity':
		return bct.modularity_und(adj,mods)[1]
	elif option=='participation coefficient':
		return bct.participation_coef(adj,mods)
	elif option=='within-module degree':
		return bct.module_degree_zscore(adj,mods)



#    (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import wx
import matplotlib
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.backends.backend_wx import NavigationToolbar2Wx

from traits.api import *
from traitsui.wx.editor import Editor
from traitsui.basic_editor_factory import BasicEditorFactory
import numpy as np
import time 

#This code is extensively adapted from Gael Varoquax's example code for
#hacking a traited matplotlib editor

class _MPLFigureEditor(Editor):

	scrollable = True
	parent = Any
	canvas = Instance(FigureCanvasWxAgg)
	tooltip = Instance(wx._misc.ToolTip)

	# define some callbacks that need to be added and removed on the fly.
	# these callbacks can't be passed around easily
	release_cid = Int
	motion_cid = Int

	waiting_for_tooltip = Bool(False)

	def init(self,parent):
		self.parent=parent
		self.control=self._create_canvas(parent)

	def update_editor(self):
		pass
		#print 'morvunskar'
		#self.reset_editor()
		#self.control=self._create_canvas(self.parent)

	def _create_canvas(self,parent):
		#unsure if there is a way to avoid hard coding these function names
		#obviously this is hacky and undesirable
		fig=self.object.circ_fig
		panel=wx.Panel(parent,-1)
		self.canvas=FigureCanvasWxAgg(panel,-1,fig)
		sizer=wx.BoxSizer(wx.VERTICAL)
		sizer.Add(self.canvas,1,wx.EXPAND|wx.ALL,1)
		#toolbar=NavigationToolbar2Wx(self.canvas)
		#toolbar.Realize()
		#sizer.Add(toolbar,0,wx.EXPAND|wx.ALL,1)
		panel.SetSizer(sizer)
		self.canvas.mpl_connect('button_press_event',
			lambda ev:self.object.circ_click(ev,self))
		self.motion_cid=self.canvas.mpl_connect('motion_notify_event',
			lambda ev:self.object.circ_mouseover(ev,self))
		self.tooltip=wx.ToolTip(tip='')
		self.tooltip.SetDelay(2000)
		self.canvas.SetToolTip(self.tooltip)
		return panel

	def _process_circ_click(self,event,cvu):
		# if the user right clicked, just display all
		if event.button==3:
			cvu.display_all()
			return
		elif event.button==2:
			self.object.mpleditor=self
			return
		# the user left clicked, lets wait and see if he wants to pan
		self.release_cid=self.canvas.mpl_connect('button_release_event',
			lambda ignore:self._single_click(event,cvu))
			# use the existing event coordinates; theres probably no difference
			# but if there were the originals would be more reliable

		#self.motion_cid=self.canvas.mpl_connect('motion_notify_event',
		#	self._pan_decide)

	def _single_click(self,event,cvu):
		self._clear_callbacks()
		#this event has xdata and ydata in reverse polar coordinates (theta,r)
		#do some algebra to figure out which ROI based on the angle
		if event.button==1 and event.ydata>=7 and event.ydata<=8:
			nod=cvu.nr_labels*event.xdata/(np.pi*2)+.5*np.pi/cvu.nr_labels
			#the formula for the correct node, assuming perfect clicking,
			#is floor(n*theta/2pi).  however, matplotlib seems to not do great
			#with this, the clicking is often too high, so i add this correction
			cvu.display_node(int(np.floor(nod)))

	def _possibly_show_tooltip(self,event,cvu):
		self._clear_callbacks()
		if event.ydata>=7 and event.ydata<=8:
			nod=int(np.floor(cvu.nr_labels*event.xdata/(np.pi*2)
				+.5*np.pi/cvu.nr_labels))
			self.tooltip.Enable(True)
			self.tooltip.SetTip(cvu.labnam[nod])
		else:
			self.tooltip.Enable(False)

		#if and when panning is also done, this logic needs to become a bit
		#more complex to respond only to novel mouse events and constantly
		#clear old callbacks.
		
		#self.waiting_for_tooltip=True
		#self.motion_cid=self.canvas.mpl_connect('motion_notify_event',
		#	self._move_unset_tooltip)
		#time.sleep(1)

	def _move_unset_tooltip(self,ignore):
		self.waiting_for_tooltip=False

	def _clear_callbacks(self):
		self.canvas.mpl_disconnect(self.release_cid)
		#self.canvas.mpl_disconnect(self.motion_cid)

	def _pan_decide(self,event):
		ax=self.canvas.figure.get_axes()[0]
		ax.set_navigate_mode('PAN')
		ax.start_pan(event.x,event.y,1)
		self._pan(event)
		self._clear_callbacks()
		self.release_cid=self.canvas.mpl_connect('button_release_event',
			self._end_pan)
		self.motion_cid=self.canvas.mpl_connect('motion_notify_event',
			self._pan)

	def _pan(self,event):
		ax = self.canvas.figure.get_axes()[0]
		ax.drag_pan(1,event.key,event.x,event.y)
		self.canvas.draw()
	
	def _end_pan(self,event):
		ax = self.canvas.figure.get_axes()[0]
		ax.end_pan()
		self._clear_callbacks()

class MPLFigureEditor(BasicEditorFactory):

	klass = _MPLFigureEditor



#    (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is copied and modified from MNE Python.  MNE Python is (C) 
#	 copyright by the authors of MNE Python 2011.  This file was modified
#	 03/2013.  For more information, see
#	 https://github.com/mne-tools/mne-python
#
#	 This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
from matplotlib.backends.backend_wxagg import *
from matplotlib.backends.backend_wx import *
from matplotlib.backends.backend_agg import *
from matplotlib import *
import wx
from mne.viz import plot_connectivity_circle
from mne.fixes import tril_indices
import pylab as pl
import matplotlib.path as m_path
import matplotlib.patches as m_patches
import matplotlib.colors as m_col
from collections import OrderedDict

#there are some minor changes in this function from the mne version
#namely the return arguments and the figure size which were hard coded in the
#mne function so i copied them

#credit largely goes to martin luessi who adapted this function to mne from
#whoever wrote it originally which is listed in the docstring
def plot_connectivity_circle2(con, nodes_numberless, indices=None, 
	n_lines=10000, node_colors=None, colormap='YlOrRd', fig=None, reqrois=[],

	node_angles=None, node_width=None, facecolor='black',
	textcolor='white', node_edgecolor='black',linewidth=1.5,
	vmin=None,vmax=None, colorbar=False, title=None):
	"""Visualize connectivity as a circular graph.

Note: This code is based on the circle graph example by Nicolas P. Rougier
http://www.loria.fr/~rougier/coding/recipes.html

This function replicates functionality from MNE python, by Martin Luessi
and others.

Parameters
----------
con : array
Connectivity scores. Can be a square matrix, or a 1D array. If a 1D
array is provided, "indices" has to be used to define the connection
indices.
nodes_numberless : list of str
Node names. The order corresponds to the order in con.
indices : tuple of arrays | None
Two arrays with indices of connections for which the connections
strenghts are defined in con. Only needed if con is a 1D array.
n_lines : int | None
If not None, only the n_lines strongest connections (strenght=abs(con))
are drawn.
node_angles : array, shape=(len(nodes_numberless,)) | None
Array with node positions in degrees. If None, the nodes are equally
spaced on the circle. See mne.viz.circular_layout.
node_width : float | None
Width of each node in degrees. If None, "360. / len(nodes_numberless)" is
used.
node_colors : list of tuples | list of str
List with the color to use for each node. If fewer colors than nodes
are provided, the colors will be repeated. Any color supported by
matplotlib can be used, e.g., RGBA tuples, named colors.
facecolor : str
Color to use for background. See matplotlib.colors.
textcolor : str
Color to use for text. See matplotlib.colors.
node_edgecolor : str
Color to use for lines around nodes. See matplotlib.colors.
linewidth : float
Line width to use for connections.
colormap : str
Colormap to use for coloring the connections.
vmin : float | None
Minimum value for colormap. If None, it is determined automatically.
vmax : float | None
Maximum value for colormap. If None, it is determined automatically.
colorbar : bool
Display a colorbar or not.
title : str
The figure title.

Returns
-------
fig : instance of pyplot.Figure
The figure handle.
"""
	n_nodes = len(nodes_numberless)

	#start_hemi = nodes_numberless[0][:3]
	#end_hemi = nodes_numberless[-1][:3]	
	#n_starthemi = sum(map(lambda lb:lb[:3]==start_hemi,nodes_numberless))
	#n_endhemi = sum(map(lambda lb:lb[:3]==end_hemi,nodes_numberless))

	if node_angles is not None:
		if len(node_angles) != n_nodes:
			raise ValueError('node_angles has to be the same length '
							 'as nodes_numberless')
		# convert it to radians
		node_angles = node_angles * np.pi / 180
	else:
		# uniform layout on unit circle
		node_angles = np.linspace(0, 2 * np.pi, n_nodes, endpoint=False)

	if node_width is None:
		node_width = 2 * np.pi / n_nodes
	else:
		node_width = node_width * np.pi / 180

	# handle 1D and 2D connectivity information
	if con.ndim == 1:
		if indices is None:
			raise ValueError('indices has to be provided if con.ndim == 1')
				#we use 1D indices
	elif con.ndim == 2:
		if con.shape[0] != n_nodes or con.shape[1] != n_nodes:
			raise ValueError('con has to be 1D or a square matrix')
		# we use the lower-triangular part
		indices = tril_indices(n_nodes, -1)
		con = con[indices]
	else:
		raise ValueError('con has to be 1D or a square matrix')

	# get the colormap
	if isinstance(colormap, basestring):
		colormap = pl.get_cmap(colormap)

	# Make figure background the same colors as axes
	if fig==None:
		fig = pl.figure(figsize=(5, 5), facecolor=facecolor)
		
	# Use a polar axes
	axes = pl.subplot(111, polar=True, axisbg=facecolor)
	#else:
		# Use the first axis already in the figure
		#axes = fig.get_axes()[0]

	# No ticks, we'll put our own
	pl.xticks([])
	pl.yticks([])

	# Set y axes limit
	pl.ylim(0, 10)
	#pl.ylim(ymin=0)

	# Draw lines between connected nodes, only draw the strongest connections
	if n_lines is not None and len(con) > n_lines:
		con_thresh = np.sort(np.abs(con).ravel())[-n_lines]
	else:
		con_thresh = 0.

	# get the connections which we are drawing and sort by connection strength
	# this will allow us to draw the strongest connections first
	con_abs = np.abs(con)
	con_draw_idx = np.where(con_abs >= con_thresh)[0]

	con = con[con_draw_idx]
	con_abs = con_abs[con_draw_idx]
	indices = [ind[con_draw_idx] for ind in indices]

	# input is already sorted
	#sort_idx = np.argsort(con_abs)
	#con_abs = con_abs[sort_idx]
	#con = con[sort_idx]
	#indices = [ind[sort_idx] for ind in indices]

	# Get vmin vmax for color scaling
	if vmin is None:
		vmin = np.min(con[np.abs(con) >= con_thresh])
	if vmax is None:
		vmax = np.max(con)
	vrange = vmax - vmin

	# We want o add some "noise" to the start and end position of the
	# edges: We modulate the noise with the number of connections of the
	# node and the connection strength, such that the strongest connections
	# are closer to the node center
	nodes_n_con = np.zeros((n_nodes), dtype=np.int)
	for i, j in zip(indices[0], indices[1]):
		nodes_n_con[i] += 1
		nodes_n_con[j] += 1

	# initalize random number generator so plot is reproducible
	rng = np.random.mtrand.RandomState(seed=0)

	n_con = len(indices[0])
	noise_max = 0.25 * node_width
	start_noise = rng.uniform(-noise_max, noise_max, n_con)
	end_noise = rng.uniform(-noise_max, noise_max, n_con)

	nodes_n_con_seen = np.zeros_like(nodes_n_con)
	for i, (start, end) in enumerate(zip(indices[0], indices[1])):
		nodes_n_con_seen[start] += 1
		nodes_n_con_seen[end] += 1

		start_noise[i] *= ((nodes_n_con[start] - nodes_n_con_seen[start])
						   / float(nodes_n_con[start]))
		end_noise[i] *= ((nodes_n_con[end] - nodes_n_con_seen[end])
						 / float(nodes_n_con[end]))

	# scale connectivity for colormap (vmin<=>0, vmax<=>1)
	con_val_scaled = (con - vmin) / vrange

	# Finally, we draw the connections
	for pos, (i, j) in enumerate(zip(indices[0], indices[1])):
		# Start point
		t0, r0 = node_angles[i], 7

		# End point
		t1, r1 = node_angles[j], 7

		# Some noise in start and end point
		t0 += start_noise[pos]
		t1 += end_noise[pos]

		verts = [(t0, r0), (t0, 5), (t1, 5), (t1, r1)]
		codes = [m_path.Path.MOVETO, m_path.Path.CURVE4, m_path.Path.CURVE4,
				 m_path.Path.LINETO]
		path = m_path.Path(verts, codes)

		color = colormap(con_val_scaled[pos])

		# Actual line
		patch = m_patches.PathPatch(path, fill=False, edgecolor=color,
									linewidth=linewidth, alpha=1.)
		axes.add_patch(patch)

	# Draw ring with colored nodes
	#radii = np.ones(n_nodes) * 8
	radii=np.ones(n_nodes)-.2
	bars = axes.bar(node_angles, radii, width=node_width, bottom=7.2,
					edgecolor=node_edgecolor, linewidth=0, facecolor='.9',
					align='center',zorder=10)

	for bar, color in zip(bars, node_colors):
		bar.set_facecolor(color)

	# Draw node labels

	#basic idea -- check for "too close" pairs.  too close is pi/50
	#remove smallest "too close" pairs.  if multiple tied within a segment,
	#remove pairs at equal spacing.
	#calculate each segment individually and find the extent of the segment.

	#TODO this parameter, too_close, could be modified and adjusted for
	#a variety of sizes if ever the circle could be panned (or if it were
	#merely made bigger).  determining the proper value is a matter of 
	#userspace testing
	too_close = np.pi/50

	# get angles for text placement
	text_angles = avgidx(nodes_numberless,n_nodes,frac=1,pad=np.pi/400)

	segments = get_tooclose_segments(text_angles,too_close,reqrois)
	
	for segment in segments:
		prune_segment(text_angles,segment,too_close)
	#TODO segments with many guaranteed ROIs are potentially spatially skewed
	#this is probably not worth fixing

	#now calculate how many pairs must be removed and remove them at equal
	#spacing.  there should be no more than theta/(n-1) >= pi/50 pairs where
	#theta is the extent and n is the number of pairs.
	#n-1 is used because each segment holds one item by default
	
	#for angles,hemi in [(text_angles_sh,start_hemi),(text_angles_eh,end_hemi)]:
	#	for name in angles:
	for name in text_angles:
		angle_rad = text_angles[name]
		#if hemi is end_hemi:
		#	angle_rad+=np.pi
		angle_deg = 180*angle_rad/np.pi
		if angle_deg >= 270 or angle_deg < 90:
			ha = 'left'
		else:
			# Flip the label, so text is always upright
			angle_deg += 180
			ha = 'right'

		name_nonum=name.strip('1234567890')
		hemi=''
		axes.text(angle_rad, 8.2, hemi+name_nonum, size=8, rotation=angle_deg,
			rotation_mode='anchor', horizontalalignment=ha,
			verticalalignment='center', color=textcolor)

	if title is not None:
		pl.subplots_adjust(left=0.2, bottom=0.2, right=0.8, top=0.75)
		pl.figtext(0.03, 0.95, title, color=textcolor, fontsize=14)
	else:
		pl.subplots_adjust(left=0.2, bottom=0.2, right=0.8, top=0.8)

	if colorbar:
		sm = pl.cm.ScalarMappable(cmap=colormap,
								  norm=pl.normalize(vmin=vmin, vmax=vmax))
		sm.set_array(np.linspace(vmin, vmax))
		ax = fig.add_axes([.92, 0.03, .015, .25])
		cb = fig.colorbar(sm, cax=ax)
		cb_yticks = pl.getp(cb.ax.axes, 'yticklabels')
		pl.setp(cb_yticks, color=textcolor)
	
	return fig

def avgidx(lbs,n,frac=.5,pad=0.):
	"""Takes:
lbs: a list of (type A) with repeats (e.g. 'supramarginal' appears 4 times)
n: where to stop
frac: fraction of 2*pi to use.  Defaults to .5 (equal length hemispheres)
pad: space to leave at the end

returns d: an ordered dictionary with (type A)/avgposition pairs
this dictionary is scaled to have values between 0 and 2*pi

example: avgidx(['A','B','B','C','D'],4)
returns OrderedDict({'A':0,'B':1.5,'C':3})"""

	d=OrderedDict()
	curlb=lbs[0]
	already=set()
	start=0 #2nd hemi needs to start at offset of too_close + end of 1st hemi
	theta=frac*(np.pi-pad)/n
	for i,e in enumerate(lbs):
		if e!=curlb:
			#get the position of the last label halfway between start and i-1
			ix=(start+i-1)*theta
			#2 cancels out from 2pi and avg of end+start/2
			while already.issuperset((curlb,)):
				#if the label is a duplicate, add meaningless numbers
				#to give it a unique spot on the circle.  We delete these
				#numbers later for display
				curlb+=str(np.random.randint(10))
			d.update({curlb:ix})
			already.add(curlb)
			#start the new label
			start=i
			curlb=e
	#add the last label
	ix=(start+i)*theta
	while already.issuperset((curlb,)):
		curlb+=str(np.random.randint(10))
	d.update({curlb:ix})
	return d

def get_tooclose_segments(angdict, too_close, required_rois=[]):
	"""Takes:
angdict: an ordered dictionary with (type A)/avgposition pairs
too_close: a float

returns a list of segments, of consecutive pairs that are closer than
too_close.  a segment contains the start item (in the dictionary), the end 
item, the extent (theta), and the number of entries.  The last index of the
segment is for required ROIs that must be displayed.

for instance: get_tooclose_segments({'A':3,'B':4,'C':5,'D':100},10)
will return [('A','C',2,3)].  There is one segment that has entries too close
together -- that from A to C.  The extent of this segment is 5-3=2.  In a real example the extent would be some fraction of 2*pi"""

	segments=[]
	keys=angdict.keys()
	nextlb=None
	start=None
	requires_here=[]

	for i,e in enumerate(angdict):
		if e in required_rois:
			requires_here.append(e)

		#try to set the next label
		if i+1<len(angdict):
			nextlb=keys[i+1]
		else:
			nextlb=None

		#check to see if we are too close to the next label
		if nextlb is not None:
			if angdict[nextlb]-angdict[e] < too_close:
				if start is None:
					#we are too close and not in a segment, start one	 
					start=i
				#move on to the next label
				continue
		
		#we aren't too close, close off the segment if needed
		if start is not None:
			extent=np.abs(angdict[e]-angdict[keys[start]])
			segment=(keys[start],keys[i],extent,i-start+1,requires_here)
			segments.append(segment)
			start=None

		#we are now looking for a new segment, so dump the ROIs collected
		requires_here=[]

	return segments

def prune_segment(angdict,seg,too_close):
	"""Remove all the labels that are too close together within a segment"""
	#print seg

	#calculate the number of labels to be removed
	extent=seg[2]
	cur_inhabitants=seg[3]
	requires=seg[4]

	max_inhabitants=1+int(np.floor(extent/too_close))
	n_removals=cur_inhabitants-max_inhabitants	

	if max_inhabitants < len(requires):
		import cvu_utils
		raise cvu_utils.CVUError('There is not enough space to display all of'
			' the ROIs that are guaranteed to be shown.  There is enough space'
			' for %i ROIs and you required %i ROIs' % (max_inhabitants,
			len(requires)))

	#remove the remaining labels, starting at the back and using equal spacing

	if cur_inhabitants/float(n_removals)>=2:
		#calculate the number of nodes to skip, before removing something
		#remove node once every "every" nodes
		every=int(np.ceil( cur_inhabitants/float(n_removals)))
	else:
		every=int(np.ceil( cur_inhabitants/(cur_inhabitants-float(n_removals))))
	#cap=int(np.ceil( cur_inhabitants/float(every)))

	keys=angdict.keys()
	vals=angdict.values()

	end=seg[1]
	start=seg[0]
	start_idx=keys.index(start)
	end_idx=keys.index(end)

	seg_dict=OrderedDict(zip(keys[start_idx:end_idx+1],
		vals[start_idx:end_idx+1]))
	#delete the entire segment and work with the temporary dict only
	for i in xrange(start_idx,end_idx+1):
		del angdict[keys[i]]

	keys=seg_dict.keys()

	start_idx=keys.index(start)#should be 0
	end_idx=keys.index(end)#should be cur_inhabitants

	start_ang=seg_dict[start]
	end_ang=seg_dict[end]

	guarantee_dict=OrderedDict()
	#remove keys not permitted to be removed
	for r in requires:
		guarantee_dict.update({r:seg_dict[r]})
		del seg_dict[r]

	keys=seg_dict.keys()
	end_idx=keys.index(end)#should be cur_inhabitants-len(requires)

	counter=0
	k=0
	#print start_idx,end_idx,n_removals
	#print seg_dict
	while counter<n_removals:
		#print "counter %i" % counter
		for i in xrange(end_idx-counter,start_idx-1,-every+k):
			#print i
			del seg_dict[keys[i]]
			counter+=1
			if counter>=n_removals:
				break
		k+=1
		keys=seg_dict.keys()

	#recalculate the angles of the remaining nodes

	seg_dict.update(guarantee_dict)

	#sort the remaining items by angle
	seg_dict=OrderedDict(sorted(seg_dict.iteritems(),
		key=lambda item:seg_dict[item[0]]))

	angs=np.linspace(start_ang,end_ang,max_inhabitants)
	keys=seg_dict.keys()

	#in theory all of this nonsense with getting the keys by keys() could be
	#made faster by passing around lots of lists and offsets.  its not pythonic
	#and probably not worthwhile (90% of optimize 10% of code etc)

	for i,theta in enumerate(angs):
		seg_dict[keys[start_idx+i]]=theta	

	#the segment is added to the end.  this is fine
	angdict.update(seg_dict)



#    (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import division
quiet=True
import cvu_utils as util
if __name__=="__main__":
	import sys
	args=util.cli_args(sys.argv[1:])
	quiet=args['quiet']
if not quiet:
	print "Importing libraries"
import numpy as np; import os
from mayavi import mlab; from mayavi.tools.animator import Animator
from traits.api import (HasTraits,Enum,Instance,Range,Float,Method,Str,Dict,
	on_trait_change,Button,Trait,TraitTuple,TraitError)
from traitsui.api import (View,VSplit,HSplit,Item,Spring,Group,ShellEditor,
	ButtonEditor,DefaultOverride)
from mayavi.core.ui.api import MlabSceneModel,MayaviScene,SceneEditor
from chaco.api import (Plot,ArrayPlotData,PlotGraphicsContext,ColorMapper,
	center)
from enable.component_editor import ComponentEditor
from chaco.tools.api import ZoomTool,PanTool
from enable.api import Pointer
from matplotlib.figure import Figure;
from matplotlib.colors import LinearSegmentedColormap
import circle_plot as circ; import mpleditor; import dialogs;
import color_legend; import color_axis; import color_map
from color_map import get_cmap_pl,set_lut
if __name__=="__main__":
	print "All libraries loaded"

class CvuPlaceholder(HasTraits):
	conn_mat = Instance(Plot)

class ConnmatPanClickTool(PanTool):
	cvu=Instance(CvuPlaceholder)

	event_state=Enum("normal","deciding","panning")
	drag_pointer=Pointer("arrow")

	def __init__(self,holder,*args,**kwargs):
		super(PanTool,self).__init__(component=holder.conn_mat,**kwargs)
		self.cvu=holder	
		
	def normal_left_down(self,event):
		self.event_state='deciding'
		self._original_xy=(event.x,event.y)
		return

	def normal_right_down(self,event):
		self.cvu.display_all()

	# the click is a node selection
	def deciding_left_up(self,event):
		self.event_state='normal'
		x,y=self.cvu.conn_mat.map_data((event.x,event.y))
		self.cvu.display_node(int(np.floor(y)))
		return

	# the click is a pan
	# the only thing of real interest in _start_pan() is change to the panning state
	def deciding_mouse_move(self,event):
		self.event_state='panning'
		return self.panning_mouse_move(event)

class Cvu(CvuPlaceholder):
	scene = Instance(MlabSceneModel, ())
	circ_fig = Instance(Figure,())

	#stateful traits -- these should be in options
	reset_thresh = Method
	display_mode = Enum('normal','scalar','module_single','module_multi')

#	node_scalars=Instance(np.ndarray)

	#buttons
	select_node_button = Button('Choose node')
	display_all_button = Button('Reset Display')
	calc_graph_button = Button('Graph Theory')
	calc_mod_button=Button('Calc modules')
	load_mod_button = Button('Load module')
	select_mod_button = Button('View module')
	custom_mod_button = Button('Custom module')
	all_mod_button = Button('View all modules')
	display_scalars_button = Button('Show scalars')
	load_scalars_button = Button('Load scalars')
	load_adjmat_button = Button('Load an adjacency matrix')
	draw_stuff_button = Button('Force render')
	color_legend_button = Button('Color legend')
	load_parc_button=Button('Load a parcellation')
	options_button=Button('Options')
	custom_colormap_button=Button('Custom Colors')
	load_track_button=Button('Load tractography')
	take_snapshot_button=Button('Take snapshot')
	make_movie_button = Button
	mk_movie_lbl = Str('Make movie')
	center_adjmat_button = Button('Center adjmat')
	about_button = Button('About')

	#various subwindows
	graph_theory_window = Instance(HasTraits)
	parc_chooser_window = Instance(HasTraits)
	adjmat_chooser_window = Instance(HasTraits)
	track_chooser_window = Instance(HasTraits)
	load_standalone_matrix_window = Instance(HasTraits)
	node_chooser_window = Instance(HasTraits)
	module_chooser_window = Instance(HasTraits)
	module_customizer_window = Instance(HasTraits)
	save_snapshot_window = Instance(HasTraits)
	configure_scalars_window = Instance(HasTraits)
	make_movie_window = Instance(HasTraits)
	really_overwrite_file_window = Instance(HasTraits)
	error_dialog_window = Instance(HasTraits)
	warning_dialog_window = Instance(HasTraits)
	about_window = Instance(HasTraits)
	color_legend_window = Instance(HasTraits)
	custom_colormap_window = Instance(HasTraits)
	opts = Instance(HasTraits)
	#Must declare each of the subwindows specifically as a trait so its 
	#attributes can be listened for as traits with decorators.

	#current display information
	cur_display_title = Str('CURRENT DISPLAY')
	cur_display_brain = Str
	cur_display_parc = Str
	cur_display_mat = Str

	default_glass_brain_color=Trait((.82,.82,.82),
		TraitTuple(Range(0.,1.),Range(0.,1.),Range(0.,1.)))

	python_shell = Dict
	

	## HAVE TRAITSUI ORGANIZE THE GUI ##
	traits_view = View(
		VSplit(
			HSplit(
				Item(name='cur_display_title',show_label=False),
				Item(name='cur_display_brain',label='subject',
					height=21,width=100),
				#Spring(),
				Item(name='cur_display_parc',label='parcellation'
					,height=21,width=100),
				#Spring(),
				Item(name='cur_display_mat',label='matrix',
					height=21,width=500),
				show_labels=True,style='readonly'),
			HSplit(
				Item(name='scene',
					editor=SceneEditor(scene_class=MayaviScene),
					height=500,width=500,show_label=False,resizable=True),
				Item(name='conn_mat',
					editor=ComponentEditor(),
					show_label=False,height=450,width=450,resizable=True),
				Group(	Item(name='select_node_button'),
						Item(name='display_all_button'),
						Item(name='color_legend_button'),
						Item(name='center_adjmat_button'),
						Spring(),
						Item(name='calc_graph_button'),
						Spring(),
						Item(name='load_scalars_button'),
						Item(name='display_scalars_button'),
						Spring(),
						Item(name='calc_mod_button'),
						Item(name='load_mod_button'),
						Item(name='select_mod_button'),
						Item(name='custom_mod_button'),
						Item(name='all_mod_button'),
						Spring(),
						Item(name='draw_stuff_button'),
					show_labels=False,
				)
			),
			HSplit(
				Item(name='circ_fig',
					editor=mpleditor.MPLFigureEditor(),
					height=500,width=500,show_label=False,resizable=True),
				Group(
				HSplit(
						Item(name='load_parc_button',),
						Item(name='load_adjmat_button',),
						Item(name='load_track_button',),
						show_labels=False,
					),
					HSplit(
						Item(name='take_snapshot_button'),
						Item(name='make_movie_button',
							editor=ButtonEditor(label_value='mk_movie_lbl')),
						Item(name='options_button',),
						Item(name='custom_colormap_button',),
						Item(name='about_button'),
						show_labels=False,
					),
					HSplit(
						Item(name='python_shell',editor=ShellEditor(),
						show_label=False),
					),
				),
			),
		),
		resizable=True,title="Connectome Visualization Utility")

	## INITIALIZE THE CVU OBJECT ##
	# args are in order pos,adj,names,srfinfo,datainfo
	def __init__(self,args):
		super(Cvu,self).__init__()
		## UNPACK THE ARG TUPLE
		self.lab_pos=args[0]
		self.adj_nulldiag=args[1]
		self.labnam=args[2]
		self.adjlabfile=args[3]
		self.srf=args[4]
		self.labv=args[5]
		self.dataloc=args[6][0]
		self.modality=args[6][1]
		self.partitiontype=args[6][2]
		self.soft_max_edges=args[6][3]

		## SET UP ALL THE DATA TO FEED TO MLAB ##
		self.nr_labels=len(self.lab_pos)

		self.cur_display_brain=args[6][4]
		self.cur_display_parc=args[6][5]
		self.cur_display_mat=args[6][6]
	
		#self.lab_pos *= 1000
		#print np.shape(self.lab_pos)
		self.opts=dialogs.OptionsWindow()
		self.graph_theory_window=dialogs.GraphTheoryWindow()
		self.custom_colormap_window=dialogs.CustomColormapWindow()
		self.adjmat_chooser_window=dialogs.AdjmatChooserWindow()
		self.parc_chooser_window=dialogs.ParcellationChooserWindow()
		self.track_chooser_window=dialogs.TractographyChooserWindow()
		self.load_standalone_matrix_window=dialogs.LoadGeneralMatrixWindow()
		self.node_chooser_window=dialogs.NodeChooserWindow()
		self.module_chooser_window=dialogs.ModuleChooserWindow()
		self.module_customizer_window=dialogs.ModuleCustomizerWindow()
		self.configure_scalars_window=dialogs.ConfigureScalarsWindow()
		self.save_snapshot_window=dialogs.SaveSnapshotWindow()
		self.make_movie_window=dialogs.MakeMovieWindow()
		self.really_overwrite_file_window=dialogs.ReallyOverwriteFileWindow()
		self.error_dialog_window=dialogs.ErrorDialogWindow()
		self.warning_dialog_window=dialogs.WarningDialogWindow()
		self.about_window=dialogs.AboutWindow()
		self.color_legend_window=color_legend.ColorLegendWindow()

		self.node_chooser_window.node_list=self.labnam
		self.module_customizer_window.initial_node_list=self.labnam

	#default initializations
	def _reset_thresh_default(self):
		return self.prop_thresh

	@on_trait_change('scene.activated')	
	def setup(self):
		## SET UP DATA ##
		self.pos_helper_gen()
		self.adj_nulldiag=util.flip_adj_ord(self.adj_nulldiag,self.adjlabfile,
			self.labnam)
		self.adj_helper_gen()

		self.node_colors_gen()

		self.curr_node=None
		self.modules=None
		self.cur_module=None

		## SET UP COLORS AND COLORMAPS ##
		ccw=self.custom_colormap_window	
		self.cmap_activation_pl=get_cmap_pl(ccw.activation_map)
		self.cmap_default_pl=get_cmap_pl(ccw.default_map)
		self.cmap_scalar_pl=get_cmap_pl(ccw.scalar_map)
		self.cmap_connmat_pl=get_cmap_pl(ccw.connmat_map)

		## SET UP ALL THE MLAB VARIABLES FOR THE SCENE ##	
		self.fig = mlab.figure(bgcolor=(.34,.34,.34),#(.36,.34,.30),
			figure=self.scene.mayavi_scene)
		self.surfs_gen()
		self.nodes_gen()
		self.vectors_gen()

		## SET UP THE CIRCLE PLOT ##
		self.circ_fig_gen()

		## SET UP CHACO VARIABLES ##
		self.chaco_gen()
		self.color_legend_gen()

		## SET UP THE CALLBACKS (for mayavi and matplotlib) ##
		pck = self.fig.on_mouse_pick(self.leftpick_callback)
		pck.tolerance = .02
		self.fig.on_mouse_pick(self.rightpick_callback,button='Right')

		self.display_all()

	## VISUALIZATION GENERATOR FUNCTIONS ##
	def init_thres_gen(self):
		self.thresval = float(
			self.adjdat[int(round(self.opts.pthresh*self.nr_edges))-1])
		if not quiet:
			print "Initial threshold: "+str(self.opts.pthresh)

	def pos_helper_gen(self,reset_scalars=True):
		self.nr_edges = self.nr_labels*(self.nr_labels-1)/2
		self.starts = np.zeros((self.nr_edges,3),dtype=float)
		self.vecs = np.zeros((self.nr_edges,3),dtype=float)
		self.edges = np.zeros((self.nr_edges,2),dtype=int)
		i=0
		for r2 in xrange(0,self.nr_labels,1):
			for r1 in xrange(0,r2,1):
				self.starts[i,:] = self.lab_pos[r1]
				self.vecs[i,:] = self.lab_pos[r2]-self.lab_pos[r1]
				self.edges[i,0],self.edges[i,1] = r1,r2
				i+=1

		#pos_helper_gen is called from three places -- setup, load_parc and 
		#load_adj. The reason it is called from load_adj is because of the soft
		#cap.  The number of edges can differ between adjmats because of the
		#soft cap and all of the positions need to be recalculated if it does.
		#Everything else in this pos_helper_gen is appropriate to run at that
		#point, except for the scalar reset.  So, I added a flag for it, for use
		#only within load_adj
		if reset_scalars:
			self.node_scalars = {}
		self.display_mode='normal'
	
	#precondition: adj_helper_gen() must be run after pos_helper_gen()
	def adj_helper_gen(self):
		self.nr_edges = int(self.nr_labels*(self.nr_labels-1)/2)
		self.adjdat = np.zeros((self.nr_edges),dtype=float)
		self.interhemi = np.zeros((self.nr_edges),dtype=bool)
		self.left = np.zeros((self.nr_edges),dtype=bool)
		self.right = np.zeros((self.nr_edges),dtype=bool)
		self.masked = np.zeros((self.nr_edges),dtype=bool)
		i=0
		for r2 in xrange(0,self.nr_labels,1):
			self.adj_nulldiag[r2][r2]=0
			for r1 in xrange(0,r2,1):
				self.adjdat[i] = self.adj_nulldiag[r1][r2]
				self.interhemi[i] = self.labnam[r1][0] != self.labnam[r2][0]
				self.left[i] = self.labnam[r1][0]==self.labnam[r2][0]=='l'
				self.right[i] = self.labnam[r1][0]==self.labnam[r2][0]=='r'
				i+=1
		self.adj_thresdiag=self.adj_nulldiag.copy()
		try:
			self.adj_thresdiag[np.where(self.adj_thresdiag==0)]=\
				np.min(self.adj_thresdiag[np.where(self.adj_thresdiag)])
		except ValueError:
			self.error_dialog("The adjmat supplied has no nonzero values")
			return

		#remove all but the soft_max_edges largest connections
		if self.nr_edges > self.soft_max_edges:
			cutoff = sorted(self.adjdat)[self.nr_edges-self.soft_max_edges-1]
			zi = np.nonzero(self.adjdat>=cutoff)
			# if way way too many edges remain, make it a hard max
			# this happens in DTI data which is very sparse, the cutoff is 0
			if len(zi[0])>(self.soft_max_edges+200):
				zi=np.nonzero(self.adjdat>cutoff)

			self.starts=self.starts[zi[0],:]
			self.vecs=self.vecs[zi[0],:]
			self.edges=self.edges[zi[0],:]
			self.adjdat=self.adjdat[zi[0]]

			self.interhemi=self.interhemi[zi[0]]
			self.left=self.left[zi[0]]
			self.right=self.right[zi[0]]
			
			self.nr_edges=len(self.adjdat)
		if not quiet:
			print str(self.nr_edges)+" total connections"

		#sort the adjdat
		sort_idx=np.argsort(self.adjdat,axis=0)
		self.adjdat=self.adjdat[sort_idx].squeeze()
		self.edges=self.edges[sort_idx].squeeze()

		self.starts=self.starts[sort_idx].squeeze() #purely cosmetic
		self.vecs=self.vecs[sort_idx].squeeze()     #purely cosmetic

		self.left=self.left[sort_idx].squeeze()
		self.right=self.right[sort_idx].squeeze()
		self.interhemi=self.interhemi[sort_idx].squeeze()
		self.masked=self.masked[sort_idx].squeeze()	#just to prune

		self.display_mode='normal'

	def surfs_clear(self):
		try:
			self.syrf_lh.remove()
			self.syrf_rh.remove()
			for child in reversed(self.fig.children):
				#reversed, to iterate over a list we remove elements from
				if (child.name=='syrfl' or child.name=='syrfr' or
						child.name=='syrfl_cracked' or 
						child.name=='syrfr_cracked'):
					self.fig.children.remove(child)
		except ValueError:
			if not quiet:
				'failed to remove old surfaces'
			pass

	def surfs_gen(self):
		ccw=self.custom_colormap_window
		self.syrf_lh = mlab.triangular_mesh(self.srf[0][:,0],self.srf[0][:,1],
			self.srf[0][:,2],self.srf[1],opacity=self.opts.surface_visibility,
			color=self.default_glass_brain_color,name='syrfl',
			colormap=ccw.scalar_map.cmap)
		self.syrf_rh = mlab.triangular_mesh(self.srf[2][:,0],self.srf[2][:,1],
			self.srf[2][:,2],self.srf[3],opacity=self.opts.surface_visibility,
			color=self.default_glass_brain_color,name='syrfr',
			colormap=ccw.scalar_map.cmap)
		#some colors
		#(.4,.75,0) #DARKISH GREEN
		#(.82,1,.82) #LIGHTER GREEN
		#(.82,.82,.82) #GRAY

		self.surfs_cracked=False
		for surf in (self.syrf_lh,self.syrf_rh):
			surf.actor.actor.pickable=0
			set_lut(surf,ccw.scalar_map)

		self.chg_lh_surfmask(); self.chg_rh_surfmask()

	def nodes_clear(self):
		try:
			self.nodesource_lh.remove()
			self.nodesource_rh.remove()
			self.txt.remove()
		except ValueError:
			pass

	def nodes_gen(self):
		ccw=self.custom_colormap_window

		#assumes that all LH nodes start with L.  This is not ideal.
		#passing this information thru preprocessing is annoying but better 
		node_hemis=np.array(map(lambda r:r[0],self.labnam))
		lhn=np.where(node_hemis=='l')[0]; self.lhnodes=lhn
		rhn=np.where(node_hemis=='r')[0]; self.rhnodes=rhn

		self.nodesource_lh = mlab.pipeline.scalar_scatter(self.lab_pos[lhn,0],
			self.lab_pos[lhn,1],self.lab_pos[lhn,2],name='nodepos_lh')
		self.nodes_lh=mlab.pipeline.glyph(self.nodesource_lh,scale_mode='none',
			scale_factor=3.0,name='nodes_lh',mode='sphere',
			colormap=ccw.default_map.cmap)
		self.nodes_lh.glyph.color_mode='color_by_scalar'

		self.nodesource_rh=mlab.pipeline.scalar_scatter(self.lab_pos[rhn,0],
			self.lab_pos[rhn,1],self.lab_pos[rhn,2],name='nodepos_rh')
		self.nodes_rh=mlab.pipeline.glyph(self.nodesource_rh,scale_mode='none',
			scale_factor=3.0,name='nodes_rh',mode='sphere',
			colormap=ccw.default_map.cmap)
		self.nodes_rh.glyph.color_mode='color_by_scalar'

		self.txt = mlab.text3d(0,0,0,'',scale=4.0,color=(.8,.6,.98,))
		self.txt.position=(0,0,83)
		self.txt.actor.actor.pickable=0

		self.chg_lh_nodemask(); self.chg_rh_nodemask()
	
		self.set_node_color_mayavi()

	def cracked_surfs_gen(self):
		ccw=self.custom_colormap_window
		tri_inds_l=[]
		tri_inds_r=[]
		for l in self.labv:
			if l.hemi=='lh':
				tris=self.srf[1]
				tri_inds=tri_inds_l
			elif l.hemi=='rh':
				tris=self.srf[3]
				tri_inds=tri_inds_r
			v_as_set=set(l.vertices)

			#get the triangles entirely contained in this set of vertices
			ts=np.where([v_as_set.issuperset(tri) for tri in tris])[0]
			tri_inds.extend(ts)

		self.syrf_lh=mlab.triangular_mesh(self.srf[0][:,0],self.srf[0][:,1],
			self.srf[0][:,2],self.srf[1][tri_inds_l],
			opacity=self.opts.surface_visibility,colormap=ccw.scalar_map.cmap,
			color=self.default_glass_brain_color,name='syrfl_cracked',)
		self.syrf_rh=mlab.triangular_mesh(self.srf[2][:,0],self.srf[2][:,1],
			self.srf[2][:,2],self.srf[3][tri_inds_r],
			opacity=self.opts.surface_visibility,colormap=ccw.scalar_map.cmap,
			color=self.default_glass_brain_color,name='syrfr_cracked')

		self.surfs_cracked=True

		for surf in (self.syrf_lh,self.syrf_rh):
			surf.actor.actor.pickable=0
			set_lut(surf,ccw.scalar_map)

		self.chg_lh_surfmask(); self.chg_rh_surfmask()

	def vectors_clear(self):
		try:
			self.vectorsrc.remove()
		except ValueError:
			pass

	def vectors_gen(self):
		ccw=self.custom_colormap_window
		self.vectorsrc = mlab.pipeline.vector_scatter(self.starts[:,0],
			self.starts[:,1],self.starts[:,2],self.vecs[:,0],self.vecs[:,1],
			self.vecs[:,2],name='connsrc')
		self.vectorsrc.mlab_source.dataset.point_data.scalars = self.adjdat 
		self.vectorsrc.outputs[0].update()
		self.init_thres_gen()
		self.thres = mlab.pipeline.threshold(self.vectorsrc,name='thresh',
			low=self.thresval)
		self.thres.auto_reset_lower=False
		self.thres.auto_reset_upper=False
		self.myvectors = mlab.pipeline.vectors(self.thres,
			colormap=ccw.activation_map.cmap,line_width=self.opts.conns_width,
			name='cons',scale_mode='vector',transparent=False)
		self.myvectors.glyph.glyph_source.glyph_source.glyph_type='dash'
		self.myvectors.glyph.glyph.clamping=False
		self.chg_conns_colors() #sets to scalar or off
		self.myvectors.actor.property.opacity=.3
		self.myvectors.actor.actor.pickable=0

		set_lut(self.myvectors,ccw.activation_map)
		self.chg_lh_connmask(); 
		self.chg_rh_connmask()
		self.chg_interhemi_connmask()
		self.chg_conns_colorbar()

	def chaco_clear(self):
		self.conn_mat.data.set_data("imagedata",np.tile(0,(self.nr_labels,
			self.nr_labels)))
		
	def chaco_gen(self):
		# set the diagonal of the adjmat to lower threshold rather than 0
		# otherwise the color scheme is a mess for non-sparse matrices
		#z_adjmat = (self.adj_thresdiag-np.mean(self.adj_thresdiag))/np.std(self.adj_thresdiag)
		#l_adjmat = np.log(self.adj_thresdiag/(1-self.adj_thresdiag))
 		#print np.where(np.isnan(l_adjmat))
		#l_adjmat[np.where(np.isnan(l_adjmat))]=0
		#print np.where(np.isnan(l_adjmat))
		self.conn_mat = Plot(ArrayPlotData(imagedata=self.adj_thresdiag))
		#centerpoint=np.mean(self.adj_thresdiag)/2+np.max(self.adj_thresdiag)/4+\
		#	np.min(self.adj_thresdiag)/4

		cm=ColorMapper.from_palette_array(self.cmap_connmat_pl(xrange(256)))
		self.conn_mat.img_plot("imagedata",name='conmatplot',colormap=cm)

		self.conn_mat.tools.append(ZoomTool(self.conn_mat))
		self.conn_mat.tools.append(ConnmatPanClickTool(self,self.conn_mat))
		self.xa=color_axis.ColorfulAxis(self.conn_mat,self.node_colors,'x')
		self.ya=color_axis.ColorfulAxis(self.conn_mat,self.node_colors,'y')
		self.conn_mat.underlays=[self.xa,self.ya]

	def circ_clear(self):
		self.circ_fig.clf()
		self.circ_fig.canvas.draw()

	# The figure parameter allows figure reuse. At startup, it should be None,
	# but if the user changes the data the existing figure should be preserved
	def circ_fig_gen(self,figure=None):
		self.circ_fig=(
			circ.plot_connectivity_circle2(
			np.reshape(self.adjdat,(self.nr_edges,)),
			self.nodes_numberless,
			indices=self.edges.T,
			colormap=self.custom_colormap_window.activation_map.cmap,
			fig=figure,
			n_lines=self.nr_edges, #bounded by soft_max_edges
			node_colors=self.node_colors,
			reqrois=self.adjmat_chooser_window.require_window.require_ls))
		self.circ_data = self.circ_fig.get_axes()[0].patches
		self.set_node_color_circ()

	def node_colors_gen(self):
		#node groups may change upon loading a new parcellation.
		#they often won't, but we have to call it at every new parcload
		hi_contrast_clist=['#26ed1a','#eaf60b','#e726f4','#002aff',
			'#05d5d5','#f4a5e0','#bbb27e','#641179','#068c40']
		hi_contrast_cmap=LinearSegmentedColormap.from_list('hi_contrast',
			hi_contrast_clist)

		#TODO don't assume that the labels have to be fs starting with 'lh_'
		self.nodes_numberless=map(
			lambda n:n.replace('div','').strip('1234567890_'),self.labnam)
		node_groups=map(lambda n:n[3:],self.nodes_numberless)

		#put group names in ordered set
		n_set=set()
		self.group_labels=[i for i in node_groups if i not in n_set 
			and not n_set.add(i)]
		self.nr_groups=len(self.group_labels)

		#get map from {node name -> node group}
		grp_ids=dict(zip(self.group_labels,xrange(self.nr_groups)))

		#group colors does not change ever
		self.group_colors=[hi_contrast_cmap(i/float(self.nr_groups))
			for i in xrange(self.nr_groups)]

		#node colors changes constantly, copy and stash the result
		self.node_colors=map(lambda n:self.group_colors[grp_ids[n]],node_groups)
		self.node_colors_default=list(self.node_colors)

		#set up some colors that are acceptably high contrast for modules
		#not related to circle plot
		self.module_colors=(
			[[255,255,255,255],[204,0,0,255],[51,204,51,255],[66,0,204,255],
			 [80,230,230,255],[51,153,255,255],[255,181,255,255],
			 [255,163,71,255],[221,221,149,255],[183,230,46,255],
			 [77,219,184,255],[255,255,204,255],[0,0,204,255],[204,69,153,255],
			 [255,255,0,255],[0,128,0,255],[163,117,25,255],[255,25,117,255]])

	def color_legend_gen(self):
		def create_entry(zipped):
			label,color=zipped
			return color_legend.LegendEntry(metaregion=label,col=color)
		self.color_legend_window.legend=\
			map(create_entry,zip(self.group_labels,self.group_colors))

	## DISPLAY LOGIC ##

	#def edge_color_on(self):
	#	self.myvectors.actor.mapper.scalar_visibility=True
	#def edge_color_off(self):
	#	self.myvectors.actor.mapper.scalar_visibility=False

	def draw_surfs(self):
		self.set_surf_color()

	def set_surf_color(self):
		#applies only to 3D brain
		csw=self.configure_scalars_window
		if self.display_mode=='scalar' and csw.srf_col:
			colors_lh=np.zeros((len(self.srf[0])))
			colors_rh=np.zeros((len(self.srf[2])))
			for i,l in enumerate(self.labv):
				if l.hemi=='lh':
					colors_lh[l.vertices]=self.node_scalars[csw.srf_col][i]
				elif l.hemi=='rh':
					colors_rh[l.vertices]=self.node_scalars[csw.srf_col][i]
			self.syrf_lh.mlab_source.scalars=colors_lh
			self.syrf_rh.mlab_source.scalars=colors_rh
			#self.opts.lh_nodes_on=False
			#self.opts.rh_nodes_on=False
			for syrf in [self.syrf_lh,self.syrf_rh]:
				syrf.actor.mapper.scalar_visibility=True
		else:
			for syrf in [self.syrf_lh,self.syrf_rh]:
				syrf.actor.mapper.scalar_visibility=False
				
	def draw_nodes(self):
		self.set_node_color()
		self.set_node_size()

	def set_node_size(self):
		#applies only to 3D brain
		csw=self.configure_scalars_window
		for nodes,idxs in [(self.nodes_lh,self.lhnodes),
				(self.nodes_rh,self.rhnodes)]:
			if self.display_mode=='scalar' and csw.nod_siz:
				nodes.glyph.scale_mode='scale_by_vector' #vector
				nodes.glyph.glyph.scale_factor=8
				sizes=self.node_scalars[csw.nod_siz][idxs]
				sizes=np.tile(sizes,(3,1)).T
				nodes.mlab_source.dataset.point_data.vectors=sizes
			else:
				nodes.glyph.scale_mode='data_scaling_off'
				nodes.glyph.glyph.scale_factor=3

	def set_node_color(self):
		if self.display_mode=='normal':
			self.node_colors=list(self.node_colors_default)
		elif self.display_mode=='scalar':
			#contract: when in scalar mode, there must always be scalars.
			self.node_colors=list(self.node_colors_default)

			#different plots might handle the scalars differently,
			#correctly we set colors as the default value and switch out
		elif self.display_mode=='module_single':
			#contract: when in module mode, there must be a current module.
			module=self.get_module()
			new_colors=np.tile(.3,self.nr_labels)
			new_colors[module]=.8
			self.node_colors=list(self.cmap_default_pl(new_colors))
		elif self.display_mode=='module_multi':
			while self.nr_modules > len(self.module_colors):
				i,j=np.random.randint(18,size=(2,))
				col=(np.array(self.module_colors[i])+self.module_colors[j])/2
				col=int(col)
				self.module_colors.append(col.tolist())
				#self.error_dialog("+18 modules not supported yet")
				#return

			#randomly permute the module colors
			perm=np.random.permutation(len(self.module_colors))
			self.module_colors=np.array(self.module_colors)[perm].tolist()
			#module colors must be saved, mayavi scalars depend on it
			
			cols=self.module_colors[:self.nr_modules]
			import bct
			ci=bct.ls2ci(self.modules,zeroindexed=True)
			self.node_colors=((np.array(self.module_colors)[ci])/255.0).tolist()

		#these routines expect self.node_colors to be set
		self.set_node_color_mayavi()
		self.set_node_color_chaco()
		self.set_node_color_circ()

	def set_node_color_mayavi(self):
		ccw=self.custom_colormap_window
		if self.display_mode=='normal':	
			for nod,nr in [(self.nodes_lh,len(self.lhnodes)),
					(self.nodes_rh,len(self.rhnodes))]:
				set_lut(nod,ccw.default_map)
				nod.mlab_source.dataset.point_data.scalars=np.tile(.3,nr)
		elif self.display_mode=='scalar':
			csw=self.configure_scalars_window
			if not csw.nod_col:
				for nod,nr in [(self.nodes_lh,len(self.lhnodes)),
						(self.nodes_rh,len(self.rhnodes))]:
					set_lut(nod,ccw.default_map)
					nod.mlab_source.dataset.point_data.scalars=np.tile(.3,nr)
			else:
				for nod,idxs in [(self.nodes_lh,self.lhnodes),
						(self.nodes_rh,self.rhnodes)]:
					set_lut(nod,ccw.scalar_map)
					nod.mlab_source.dataset.point_data.scalars=(	
						self.node_scalars[csw.nod_col][idxs])
		elif self.display_mode=='module_single':
			for nod,idxs in [(self.nodes_lh,self.lhnodes),
					(self.nodes_rh,self.rhnodes)]:
				set_lut(nod,ccw.default_map)
				new_colors=np.tile(	.3,self.nr_labels)
				new_colors[self.get_module()]=.8
				nod.mlab_source.dataset.point_data.scalars=new_colors[idxs]
		elif self.display_mode=='module_multi':
			cols=np.array(self.module_colors)[:self.nr_modules]
			
			#set the cmap to be relatively discrete colors
			for nod in [self.nodes_lh,self.nodes_rh]:
				#we are going to override lut_mode and set the LUT manually.
				#when we want to go back to some other mode lut_mode is still
				#the same it will not trigger a change event.
				#black-white is disallowed as a user-specified value so we
				#specify this as a placeholder
				nod.module_manager.scalar_lut_manager.lut_mode='black-white'
				nod.module_manager.scalar_lut_manager.number_of_colors=(
					self.nr_modules)
				nod.module_manager.scalar_lut_manager.lut.table=cols

			#set the mlab scalars to be fractions between 0 and 1 for the cmap
			import bct
			for nodes,idxs in [(self.nodes_lh,self.lhnodes),
					(self.nodes_rh,self.rhnodes)]:
				nodes.mlab_source.dataset.point_data.scalars=(bct.ls2ci(
					self.modules,zeroindexed=True)/self.nr_modules)[idxs]
				
		mlab.draw()

	def set_node_color_circ(self):
		csw=self.configure_scalars_window
		if self.display_mode=='scalar' and csw.circle:
			cols=list(self.cmap_scalar_pl(self.node_scalars[csw.circle]))
		else:
			cols=self.node_colors

		#the circle only plots the ~max_edges highest edges.  the exact number 
		#varies with the data, but we can inspect a variable to find it
		circ_path_offset=len(self.adjdat)
		for n in xrange(0,self.nr_labels,1):
			self.circ_data[circ_path_offset+n].set_fc(cols[n])
		self.circ_fig.canvas.draw()

	def set_node_color_chaco(self):
		csw=self.configure_scalars_window
		if self.display_mode=='scalar' and csw.conmat:
			cols=list(self.cmap_scalar_pl(self.node_scalars[csw.conmat]))
		else:
			cols=self.node_colors

		self.xa.colors = cols
		self.ya.colors = cols
		self.conn_mat.request_redraw()

	def draw_conns(self):
		self.set_conns_active()

	def set_conns_active(self):
		self.reset_thresh()
		lo=self.thres.lower_threshold
		hi=self.thres.upper_threshold

		def select_connections(conditions):
			new_edges = np.zeros([self.nr_edges,2],dtype=int)
			count_edges = 0
			for e,(a,b) in enumerate(zip(self.edges[:,0],
					self.edges[:,1])):
				#if not self.masked[e] and (self.curr_node is None or
				#		self.curr_node in [a,b]):
				if conditions(e,a,b):
					new_edges[e]=(a,b)

					#do operations requiring threshold checking
					#done here for efficiency.
					#this is the high usage bit of code to optimize
					if (self.adjdat[e] <= hi and
							self.adjdat[e] >= lo):
						self.circ_data[e].set_visible(True)
						col=self.cmap_activation_pl((self.adjdat[e]-lo)/(hi-lo))
						self.circ_data[e].set_ec(col)
						count_edges+=1
					else:
						self.circ_data[e].set_visible(False)
				else:
					new_edges[e]=(0,0)
					self.circ_data[e].set_visible(False)
			return new_edges,count_edges

		basic_conds = lambda e,a,b:(not self.masked[e] and
			(self.curr_node is None or self.curr_node in [a,b]))

		if self.display_mode=='module_single':
			#find the right module
			module=self.get_module()
			#attach the right conditions
			if self.opts.module_view_style=='intramodular':
				conds = lambda e,a,b:(basic_conds(e,a,b) and
					(a in module and b in module))
			elif self.opts.module_view_style=='intermodular':
				conds = lambda e,a,b:(basic_conds(e,a,b) and
					((a in module) != (b in module))) #xor
			elif self.opts.module_view_style=='both':
				conds = lambda e,a,b:(basic_conds(e,a,b) and
					(a in module or b in module))
		else:
			conds=basic_conds

		new_edges,count_edges=select_connections(conds)
		
		new_starts=self.lab_pos[new_edges[:,0]]
		new_vecs=self.lab_pos[new_edges[:,1]] - new_starts
		
		self.vectorsrc.mlab_source.reset(
			x=new_starts[:,0],y=new_starts[:,1],z=new_starts[:,2],
			u=new_vecs[:,0],v=new_vecs[:,1],w=new_vecs[:,2])
	
		if self.curr_node is not None:
			self.myvectors.actor.property.opacity=.75
			self.txt.set(text='  '+self.labnam[self.curr_node])
		else:
			self.myvectors.actor.property.opacity=.3
			self.txt.set(text='')

		if not quiet and self.curr_node is not None:
			print ("node %i: %s, expecting %i edges" % 
				(self.curr_node, self.labnam[self.curr_node], count_edges))

		#TODO CHACO

		mlab.draw()
		self.circ_fig.canvas.draw()

	def get_module(self):
		if self.cur_module=='custom':
			return self.custom_module
		if type(self.cur_module)==int:
			return self.modules[self.cur_module]
		else:
			self.error_dialog('Internal error: Current module not recognized')

	## FUNCTIONS FOR LOADING NEW DATA ##
	def load_new_parcellation(self):
		try:
			pcw=self.parc_chooser_window
			labnam,ign = util.read_parcellation_textfile(pcw.labelnames_f)
			labv = util.loadannot(pcw.parcellation_name,pcw.SUBJECT,
				pcw.SUBJECTS_DIR,self.srf[4])
			self.lab_pos,self.labv = util.calcparc(labv,labnam,quiet=quiet,
				parcname=pcw.parcellation_name)
			self.srf = util.loadsurf(os.path.join(pcw.SUBJECTS_DIR,pcw.SUBJECT,
				'surf','lh.%s'%self.srf[4]),self.srf[4])
			self.cur_display_brain=pcw.SUBJECT
			self.cur_display_parc=pcw.parcellation_name
			self.cur_display_mat=''
		except IOError as e:
			self.error_dialog(str(e))
			return
		self.labnam=labnam
		self.node_chooser_window.node_list=labnam
		self.module_customizer_window.initial_node_list=labnam
		self.nr_labels=len(self.labnam)
		self.pos_helper_gen()
		print "Parcellation %s loaded successfully" % pcw.parcellation_name
		self.surfs_clear()
		self.nodes_clear()
		self.vectors_clear()
		self.chaco_clear()
		self.circ_clear()
		self.nodes_gen()
		self.surfs_gen()
		self.node_colors_gen()
		self.chg_scalar_colorbar()
		#scalar colorbar loading is tied to the surface and not to nodes
		#because the surface always has the same color scheme and the nodes
		#don't.  but it can't be in surfs_gen because the surf can get gen'd
		#when switching from cracked to glass. so it is here.
	
	def load_new_adjmat(self):
		acw=self.adjmat_chooser_window
		try:
			adj=util.loadmat(acw.adjmat,field=acw.field_name)
			if acw.adjmat_order:
				self.adjlabfile=acw.adjmat_order
				adj=util.flip_adj_ord(adj,self.adjlabfile,self.labnam,
					ign_dels=acw.ignore_deletes)
			if acw.max_edges>0:
				self.soft_max_edges=acw.max_edges
			self.cur_display_mat=acw.adjmat
		except (util.CVUError,IOError) as e:
			self.error_dialog(str(e))
			return
		except (ValueError,IndexError) as e:
			self.error_dialog("Mismatched channels: %s" % str(e))
			raise
			#return
		except KeyError as e:
			self.error_dialog("Field not found: %s" % str(e))
			return
		if len(adj) != self.nr_labels:
			self.error_dialog('The adjmat specified is of size %i and the '
				'parcellation size is %i' %(len(adj),self.nr_labels))
			return
		self.adj_nulldiag = adj
		print "Adjacency matrix %s loaded successfully" % acw.adjmat
		#it is necessary to rerun pos_helper_gen() because the number of edges
		#is not constant from one adjmat to another (soft cap). 
		#see comment in pos_helper_gen
		self.pos_helper_gen(reset_scalars=False)
		self.adj_helper_gen()
		self.curr_node=None
		self.cur_module=None
		self.txt.set(text='')
		self.init_thres_gen()
		self.vectors_clear()
		self.vectors_gen()
		self.chaco_gen()
		self.circ_clear()
		self.circ_fig_gen(figure=self.circ_fig)
		self.color_legend_gen()	
		self.draw_surfs() #for surf color
		self.draw_nodes()
		self.draw_conns()

	def load_tractography(self):
		tcw=self.track_chooser_window
		try:
			import tractography
			self.tractography=tractography.plot_fancily(tcw.track_file)
			tractography.apply_cmp_affines(self.tractography,tcw.b0_volume,
				tcw.track_file,tcw.SUBJECT,tcw.SUBJECTS_DIR,
				fsenvsrc=tcw.fs_setup)
		except Exception as e:
			self.error_dialog(str(e))

	def load_standalone_matrix(self):
		lsmw=self.load_standalone_matrix_window
		if not lsmw.dataset_name:
			self.error_dialog("Cannot leave dataset name blank.  cvu uses "
				"this value to keep track of the data.")
			return
		try:
			ci=util.loadmat(lsmw.mat,field=lsmw.field_name)
			if lsmw.mat_order:
				init_ord,bads=util.read_parcellation_textfile(lsmw.mat_order)
				#delete the extras
				if not lsmw.ignore_deletes:
					ci=np.delete(ci,bads)
				ci_ord=util.adj_sort(init_ord,self.labnam)
				#swap the new order
				ci=ci[ci_ord]
		except (util.CVUError,IOError) as e:
			self.error_dialog(str(e))
			return
		except (ValueError,IndexError) as e:
			self.error_dialog("Mismatched channels: %s" % str(e))
			return
		except KeyError as e:
			self.error_dialog("Field not found: %s" % str(e))
			return

		try:
			#make sure the order is the right size
			ci=np.reshape(ci,(self.nr_labels,))
		except ValueError as e:
			self.error_dialog("Matrix loaded is of wrong size.  Expected "
				"size (%i,1) and got %s" % (self.nr_labels,str(np.shape(ci))))
			
		if lsmw.whichkind=='modules':
			import bct
			self.modules=bct.ci2ls(ci)
			self.update_modules_metadata()
		elif lsmw.whichkind=='scalars':
			ci=(ci-np.min(ci))/(np.max(ci)-np.min(ci))
			#normalize scalars to 0-1 range
			self.node_scalars.update({lsmw.dataset_name:ci})

		lsmw.dataset_plusplus()

	## BASIC VISUALIZATION INTERACTIONS ##
	def error_dialog(self,message):
		self.error_dialog_window.error=message
		self.error_dialog_window.edit_traits()

	def warning_dialog(self,message):
		self.warning_dialog_window.warning=message
		self.warning_dialog_window.edit_traits()

	@on_trait_change('display_all_button')
	def display_all(self):
		self.display_mode='normal' #no?  user needs some other way to reset this
		self.curr_node=None
		self.cur_module=None #no?

		self.draw_surfs()
		self.draw_nodes()
		self.draw_conns()

	def display_node(self,n):
		if n<0 or n>=self.nr_labels:
			return
		self.curr_node=n
		self.draw_conns()

	def display_scalars(self):
		self.display_mode='scalar'
		self.draw_surfs()
		self.draw_nodes()

	def display_module(self,module):
		self.display_mode='module_single'
		self.curr_node=None
		self.cur_module=module

		self.draw_surfs() #needed to unset surf color in edge case
		self.draw_nodes()
		self.draw_conns()

		if not quiet:
			print "%i nodes in module" % len(self.get_module())

	@on_trait_change('all_mod_button')
	def display_multi_module(self):
		if self.modules is None:
			self.error_dialog('No modules defined')
			return
		self.display_mode='module_multi'
		self.draw_nodes()

	#node selection
	def _select_node_button_fired(self):
		self.node_chooser_window.cur_node=-1
		self.node_chooser_window.edit_traits()

	@on_trait_change('node_chooser_window:notify')
	def node_select_check(self):
		ncw=self.node_chooser_window
		if (not ncw.finished) or ncw.cur_node==-1:
			pass
		else:
			self.display_node(ncw.cur_node)

	#scalar selection
	def _display_scalars_button_fired(self):
		if not self.node_scalars:
			self.error_dialog('No scalars defined')
			return
		csw=self.configure_scalars_window
		csw.finished=False
		csw.scalar_sets=self.node_scalars.keys()
		csw.nod_col='';csw.srf_col='';csw.nod_siz='';csw.circle='';csw.conmat=''
		csw.edit_traits()

	@on_trait_change('configure_scalars_window:notify')
	def scalar_select_check(self):
		csw=self.configure_scalars_window
		if (not csw.finished) or (not any([csw.nod_col,csw.srf_col,csw.nod_siz,
				csw.circle,csw.conmat])):
			pass
		else:
			self.display_scalars()

	#graph theory
	def _calc_graph_button_fired(self):
		gtw=self.graph_theory_window
		#first check if stats are computed, if so let user ask for recalculate
		if not gtw.graph_stats:
			self.calc_graphstats()
		#now, check to see if there are any, just in case 0 are specified
		if gtw.graph_stats:
			gtw.current_stat=gtw.graph_stats[0]
		self.graph_theory_window.edit_traits()
		
	@on_trait_change('graph_theory_window:RecalculateEvent')
	def calc_graphstats(self):
		import graph,bct
		try: 
			stats=graph.do_summary(self.adj_nulldiag,bct.ls2ci(self.modules),
				self.opts.intermediate_graphopts_list)
		except util.CVUError:
			self.error_dialog("Community structure required for some " 
				"of the calculations specified. Try calculating modules first.")
			return

		self.graph_theory_window.graph_stats=list()
		for k,v in stats.iteritems():
			self.graph_theory_window.graph_stats.append(
				graph.StatisticsDisplay(k,v,self.labnam))

	@on_trait_change('graph_theory_window:SaveToScalarEvent')
	def sv_graphstat_to_scalar(self):
		gtw=self.graph_theory_window
		if np.shape(gtw.current_stat.stat)!=(self.nr_labels,1):
			self.error_dialog("Only Nx1 vectors can be saved as scalars")
			return
		ci=gtw.current_stat.stat.ravel().copy()
		ci=(ci-np.min(ci))/(np.max(ci)-np.min(ci))
		self.node_scalars.update({gtw.scalar_savename:ci})

	#module selection
	def _calc_mod_button_fired(self):
		import bct
		if self.partitiontype=="spectral":
			self.modules=bct.ci2ls(bct.modularity_und(self.adj_nulldiag)[0])
		else:
			raise Exception("Partition type %s not found" % self.partitiontype)
		self.update_modules_metadata()

	def update_modules_metadata(self):
		#separated out because this code is called in two separate instances
		if self.modules is None:
			self.error_dialog('Modules were not loaded properly')
			return
		self.nr_modules=len(self.modules)
		self.module_chooser_window.module_list=[]
		for i in xrange(0,self.nr_modules):
			self.module_chooser_window.module_list.append('Module '+str(i))

	def _select_mod_button_fired(self):
		if self.modules is None:
			self.error_dialog('No modules defined')
		else:
			self.module_chooser_window.cur_mod=-1
			self.module_chooser_window.finished=False
			self.module_chooser_window.edit_traits()

	@on_trait_change('module_chooser_window:notify')
	def mod_select_check(self):
		mcw=self.module_chooser_window
		if not mcw.finished or mcw.cur_mod==-1:
			pass
		else:
			self.display_module(mcw.cur_mod)

	def _custom_mod_button_fired(self):
		self.module_customizer_window.finished=False
		self.module_customizer_window.edit_traits()

	@on_trait_change('module_customizer_window:notify')
	def custom_mod_check(self):
		mcw=self.module_customizer_window
		if not mcw.finished:
			pass
		else:
			try:
				mcw.index_convert()
			except ValueError as e:
				self.error_dialog('Something went wrong! Blame the programmer')
			self.custom_module=mcw.return_module
			#self.cur_module='custom'
			self.display_module('custom')

	## CALLBACKS ##
	#chaco callbacks are in ConnmatPointSelector class out of necessity
	#where they override the _select method
	def leftpick_callback(self,picker):
		#for actor in picker.actors
			#if actor in self.nodes.actor.actors:
			#	correct_actor=actor
			#ptid
		for nodes in [self.nodes_lh,self.nodes_rh]:
			if picker.actor in nodes.actor.actors:
				ptid=(picker.point_id/nodes.glyph.glyph_source.glyph_source.
					output.points.to_array().shape[0])
				if ptid != -1 and nodes is self.nodes_lh:
					self.display_node(self.lhnodes[int(ptid)])
				elif ptid != -1 and nodes is self.nodes_rh:
					self.display_node(self.rhnodes[int(ptid)])
		self.pick=picker

	def rightpick_callback(self,picker):
		self.display_all()

	def circ_click(self,event,mpledit):
		if not quiet:
			print 'button=%d,x=%d,y=%d,xdata=%s,ydata=%s'%(event.button,event.x,
				event.y,str(event.xdata),str(event.ydata))
		# in principle all the clicking logic would be done here, but i felt
		# this file was cluttered enough
		mpledit._process_circ_click(event,self)

	def circ_mouseover(self,event,mpledit):
		mpledit._possibly_show_tooltip(event,self)

	## MISCELLANEOUS OPTIONS ##
	#one possible value of self.reset_thresh()
	def prop_thresh(self):	
		self.thresval=float(
			self.adjdat[int(round(self.opts.pthresh*self.nr_edges))-1])
		dmax=self.adjdat[self.nr_edges-1]
		self.thres.set(upper_threshold=dmax,lower_threshold=self.thresval)
		if not quiet:
			print "upper threshold "+("%.4f" % self.thres.upper_threshold)
			print "lower threshold "+("%.4f" % self.thres.lower_threshold)

	#another possible method for self.reset_thresh()
	def num_thresh(self):
		if self.opts.thresh_type!='num':
			return
		try:
			self.thresval=self.opts.nthresh
			dmax=float(self.adjdat[self.nr_edges-1])
			self.thres.set(upper_threshold=dmax,lower_threshold=self.thresval)
		except TraitError as e:
			self.error_dialog(str(e))
			return
		self.vectorsrc.outputs[0].update()
		if not quiet:
			print "upper threshold "+("%.4f" % self.thres.upper_threshold)
			print "lower threshold "+("%.4f" % self.thres.lower_threshold)

	#keep separated the drawing logic from the threshold logic, so that we can
	#just call draw on changing and always reset the threshold on drawing	
	@on_trait_change('opts:pthresh')
	def chg_pthresh_val(self):
		if self.opts.thresh_type != 'prop':
			return
		self.draw_conns()

	@on_trait_change('opts:nthresh')
	def chg_nthresh_val(self):
		if self.opts.thresh_type != 'num':
			return
		self.draw_conns()

	@on_trait_change('opts:thresh_type')
	def chg_thresh_type(self):
		if self.opts.thresh_type=='prop':
			self.reset_thresh=self.prop_thresh
		elif self.opts.thresh_type=='num':
			self.reset_thresh=self.num_thresh
		self.draw_conns() #will call reset_thresh()

	@on_trait_change('opts:surface_visibility')
	def chg_syrf_vis(self):
		self.syrf_lh.actor.property.set(opacity=self.opts.surface_visibility)
		self.syrf_rh.actor.property.set(opacity=self.opts.surface_visibility)

	@on_trait_change('opts:circ_size')
	def chg_circ_size(self):
		self.circ_fig.axes[0].set_ylim(0,self.opts.circ_size)
		#self.redraw_circ()

	@on_trait_change('opts:show_floating_text')
	def chg_float_text(self):
		self.txt.visible=self.opts.show_floating_text

	# beware.  currently masking only comes from one of these three types
	# which are mutually exclusive.  if this changes, xor wont work anymore
	# one thing that would work would be using addition of binary flags as types
	# but better to make three sets of conns so as not to deal with this mess
	@on_trait_change('opts:interhemi_conns_on')
	def chg_interhemi_connmask(self):
		self.masked[self.interhemi]=not self.opts.interhemi_conns_on
		
	@on_trait_change('opts:lh_conns_on')
	def chg_lh_connmask(self):
		self.masked[self.left]=not self.opts.lh_conns_on
	
	@on_trait_change('opts:rh_conns_on')
	def chg_rh_connmask(self):
		self.masked[self.right]=not self.opts.rh_conns_on

	@on_trait_change('opts:lh_nodes_on')
	def chg_lh_nodemask(self):
		self.nodes_lh.visible=self.opts.lh_nodes_on

	@on_trait_change('opts:rh_nodes_on')
	def chg_rh_nodemask(self):
		self.nodes_rh.visible=self.opts.rh_nodes_on

	@on_trait_change('opts:lh_surfs_on')
	def chg_lh_surfmask(self):
		self.syrf_lh.visible=self.opts.lh_surfs_on

	@on_trait_change('opts:rh_surfs_on')
	def chg_rh_surfmask(self):
		self.syrf_rh.visible=self.opts.rh_surfs_on

	@on_trait_change('opts:conns_width')
	def chg_conns_width(self):
		self.myvectors.actor.property.line_width=self.opts.conns_width

	@on_trait_change('opts:conns_colors_on')
	def chg_conns_colors(self):
		if self.opts.conns_colors_on:
			self.myvectors.glyph.color_mode='color_by_scalar'
		else:
			self.myvectors.glyph.color_mode='no_coloring'

	@on_trait_change('opts:conns_colorbar')
	def chg_conns_colorbar(self):
		v=self.myvectors
		if self.opts.conns_colorbar:
			mlab.colorbar(object=v,orientation='horizontal',title='')
		else:
			v.module_manager.scalar_lut_manager.show_scalar_bar=False

	@on_trait_change('opts:scalar_colorbar')
	def chg_scalar_colorbar(self):
		s=self.syrf_lh
		if self.opts.scalar_colorbar:
			mlab.colorbar(object=s,orientation='vertical',title='')
		else:
			s.module_manager.scalar_lut_manager.show_scalar_bar=False

	@on_trait_change('opts:render_style')
	def chg_render_style(self):
		#doesnt make sense to change the visualization if the surfaces are off
		self.opts.lh_surfs_on=True
		self.opts.rh_surfs_on=True

		if self.surfs_cracked:
			if self.opts.render_style=='cracked glass':
				return
			else:
				self.surfs_clear()
				self.surfs_gen()
				self.set_surf_color()
		if self.opts.render_style=='cracked_glass':
			self.surfs_clear()
			self.cracked_surfs_gen()
			self.set_surf_color()
		else:
			for syrf in [self.syrf_lh,self.syrf_rh]:
				if self.opts.render_style=='contours':
					syrf.actor.property.representation='surface'
					syrf.enable_contours=True
					syrf.contour.number_of_contours=250	
				else:
					syrf.enable_contours=False
				if self.opts.render_style=='glass':
					syrf.actor.property.representation='surface'
				elif self.opts.render_style=='wireframe':
					syrf.actor.property.representation='wireframe'
				elif self.opts.render_style=='speckled':
					syrf.actor.property.representation='points'

	@on_trait_change('custom_colormap_window:default_map.+')
	def chg_default_cmap_interactive(self):
		ccw=self.custom_colormap_window
		self.cmap_default_pl=get_cmap_pl(ccw.default_map)

		try:
			self.draw_surfs()
			self.draw_nodes()
		except: 
			if ccw.default_map.cmap=='file' and not ccw.fname_default.fname:
				pass #fail silently until user has a chance to specify file
			else: raise

	@on_trait_change('custom_colormap_window:scalar_map.+')
	def chg_scalar_cmap_interactive(self):
		ccw=self.custom_colormap_window

		#surf color doesnt change anywhere else; keep out of draw for simplicity
		for surf in [self.syrf_lh,self.syrf_rh]:
			set_lut(surf,ccw.scalar_map)

		self.cmap_scalar_pl=get_cmap_pl(ccw.scalar_map)

		try:
			self.draw_surfs()
			self.draw_nodes()
		except:
			if ccw.scalar_map.cmap=='file' and not ccw.scalar_map.fname: pass
			else: raise

	@on_trait_change('custom_colormap_window:activation_map.+')
	def chg_activation_cmap_interactive(self):
		ccw=self.custom_colormap_window

		#conncolor doesnt change anywhere else; keep out of draw for simplicity
		set_lut(self.myvectors,ccw.activation_map)

		self.cmap_activation_pl=get_cmap_pl(ccw.activation_map)

		try:
			self.draw_conns()
		except:
			if ccw.activation_map.cmap=='file' and not ccw.activation_map.fname:
				pass #fail silently until user has a chance to specify file
			else:
				raise
	
	@on_trait_change('custom_colormap_window:connmat_map.+')
	def chg_connmat_cmap_interactive(self):
		ccw=self.custom_colormap_window
		self.cmap_connmat_pl=get_cmap_pl(ccw.connmat_map)

		#chaco plot color doesnt change anywhere else ever
		
		try:
			cm=self.cmap_connmat_pl(xrange(256)) #colormap num_colors=256
			self.conn_mat.color_mapper = ColorMapper.from_palette_array(cm)
			self.conn_mat.request_redraw()
		except:
			if ccw.connmat_map.cmap=='file' and not ccw.connmat_map.fname:
				pass #fail silently until user has a chance to specify file
			else:
				raise


	@on_trait_change('custom_colormap_window:notify')
	def colormap_customize_check(self):
		ccw=self.custom_colormap_window

		errstr=''

		for map in [ccw.default_map,ccw.scalar_map,ccw.activation_map,
				ccw.connmat_map]:
			if map.cmap=='file' and not map.fname:
				errstr+='No file specified for %s\n' % map.label
				map.reset_traits(['cmap','reverse'])

		if errstr:
			self.error_dialog(errstr)

	## LOAD DATA HELPER FUNCTIONS ##
	def _load_adjmat_button_fired(self):
		self.adjmat_chooser_window.finished=False
		self.adjmat_chooser_window.edit_traits()

	@on_trait_change('adjmat_chooser_window:notify')
	def load_adjmat_check(self):
		acw=self.adjmat_chooser_window
		if not acw.finished:
			pass
		elif acw.adjmat:
			self.load_new_adjmat()
		else:
			self.error_dialog('You must specify the adjacency matrix')

	def _load_parc_button_fired(self):
		self.parc_chooser_window.finished=False
		self.parc_chooser_window.edit_traits()

	@on_trait_change('parc_chooser_window:notify')
	def load_parc_check(self):
		pcw=self.parc_chooser_window
		if not pcw.finished:
			pass
		elif (pcw.SUBJECTS_DIR and pcw.SUBJECT and pcw.parcellation_name and
				pcw.labelnames_f):
			self.load_new_parcellation()
		else:
			self.error_dialog('You must specify all of SUBJECT, SUBJECTS_DIR, '
				'the desired label ordering, and the parcellation name '
				'(e.g. aparc.2009)')

	def _load_track_button_fired(self):
		self.track_chooser_window.finished=False
		self.track_chooser_window.edit_traits()

	@on_trait_change('track_chooser_window:notify')
	def load_track_check(self):
		tcw=self.track_chooser_window
		if not tcw.finished:
			pass
		elif (tcw.track_file and tcw.b0_volume and tcw.SUBJECTS_DIR and
				tcw.SUBJECT):
			if self.cur_display_brain=='fsavg5':
				self.warning_dialog('Current surface is fsaverage5. '
					'Tractography is misaligned without subject morphology') 
			self.load_tractography()
		else:
			self.error_dialog('You must specify all of SUBJECT, SUBJECTS_DIR, '
				'the .trk file, and the B0 volume')

	def _load_mod_button_fired(self):
		self.load_standalone_matrix_window.finished=False
		self.load_standalone_matrix_window.whichkind='modules'
		self.load_standalone_matrix_window.edit_traits()

	def _load_scalars_button_fired(self):
		self.load_standalone_matrix_window.finished=False
		self.load_standalone_matrix_window.whichkind='scalars'
		self.load_standalone_matrix_window.edit_traits()

	@on_trait_change('load_standalone_matrix_window:notify')
	def load_standalone_matrix_check(self):
		lmw=self.load_standalone_matrix_window
		if not lmw.finished:
			pass
		elif lmw.mat:
			#check whichkind in the load_standalone() function,
			#in both cases the ordering procedure is the same
			self.load_standalone_matrix()
		else:
			self.error_dialog('You must specify a valid matrix file')	
		
	## MAKE SNAPSHOTS AND MOVIES ##
	def _take_snapshot_button_fired(self):
		self.save_snapshot_window.finished=False
		self.save_snapshot_window.edit_traits()

	@on_trait_change('save_snapshot_window:notify')
	def save_snapshot_check(self):
		ssw=self.save_snapshot_window
		if not ssw.finished:
			pass
		# use continuation passing style
		elif ssw.savefile:
			# capture the continuation
			def saveit():
				try:
					# the contents of the continuation depend on the plot type
					if ssw.whichplot=='circle plot':
						self.circ_fig.savefig(ssw.savefile,dpi=ssw.dpi,
							facecolor='black')
					elif ssw.whichplot=='3D brain':
						res=np.ceil(500*ssw.dpi/8000.0*111)
						self.hack_mlabsavefig(ssw.savefile,size=(res,res))
					elif ssw.whichplot=='connection matrix':
						gc=PlotGraphicsContext(self.conn_mat.outer_bounds,
							dpi=ssw.dpi)
						gc.render_component(self.conn_mat)
						gc.save(ssw.savefile)
				except IOError as e:
					self.error_dialog(str(e))
				except KeyError as e:
					self.error_dialog("The library handling the operation you "
						"requested supports multiple file types.  Please "
						"specify a file extension to disambiguate.")
			if os.path.exists(ssw.savefile):
				# set the continuation on the rofw widget and spawn it
				self.set_save_continuation_and_spawn_rofw(saveit)
			else:
				# not a duplicate filename, just call the save continuation
				saveit()
		else:
			self.error_dialog('You must specify the save file')

	def set_save_continuation_and_spawn_rofw(self,continuation):
		self.really_overwrite_file_window.finished=False
		self.really_overwrite_file_window.save_continuation=continuation
		self.really_overwrite_file_window.edit_traits()

	@on_trait_change('really_overwrite_file_window:notify')
	def really_overwrite_file_check(self):
		rofw=self.really_overwrite_file_window
		# if the user clicks OK, call the save continuation
		if rofw.finished:
			rofw.save_continuation()
		# otherwise, don't do anything

	def hack_mlabsavefig(self,fname,size):
		oldx,oldy=self.scene.scene_editor.get_size()

		curx,cury=self.scene.scene_editor.control.Parent.Parent.Size
		cury-=32 #the mayavi toolbar takes up 32 space, which is not what
				 #we are setting

		self.scene.scene_editor.set_size((curx,cury))
		self.txt.visible=False

		#TODO remove: this bug is fixed in mayavi 4.3
		magnif_desired = max(size[0]//curx,size[1]//cury)+1
		newsize=(int(size[0]/magnif_desired),int(size[1]/magnif_desired))
		self.scene.scene_editor.set_size(newsize)
		from tvtk.api import tvtk
		filter=tvtk.WindowToImageFilter(read_front_buffer=True)
		filter.magnification=int(magnif_desired)
		self.scene.scene_editor._lift()
		filter.input = self.scene.scene_editor._renwin
		ex = tvtk.PNGWriter()
		ex.file_name = fname
		ex.input = filter.output
		self.scene.scene_editor._exporter_write(ex)

		#reset the old size
		self.scene.scene_editor.set_size((oldx,oldy))
		#restore the 3D region text if it was previously enabled
		self.txt.visible=self.opts.show_floating_text

	def _make_movie_button_fired(self):
		if self.mk_movie_lbl=='Make movie':
			self.make_movie_window.finished=False
			self.make_movie_window.edit_traits()
		else:
			self.do_mkmovie_finish()

	@on_trait_change('make_movie_window:notify')
	def make_movie_check(self):
		mmw=self.make_movie_window
		self.animator=None
		self.curdir=None
		if not mmw.finished:
			pass
		else:
			def saveit():
				self.mk_movie_lbl='Stop movie'
				if mmw.type=='x11grab':
					self.do_mkmovie_x11grab()
				else:
					self.do_mkmovie_snapshots(mmw.samplerate,mmw.anim_style)
			if os.path.exists(mmw.savefile):
				self.set_save_continuation_and_spawn_rofw(saveit)
			else:
				saveit()

	def do_mkmovie_animate(self,samplerate,rotate_on,snapshot_folder=None):
		if not snapshot_folder and not rotate_on:
			return
		def anim():
			i=0
			while True:
				if snapshot_folder:
					fname=os.path.join(snapshot_folder,'/movie%05d.png' % i)
					mlab.savefig(fname)
					i+=1
				if rotate_on:
					self.scene.camera.azimuth(10)
					self.scene.render()
				yield
		animation=anim()
		fps_in=int(1000/samplerate) #result is in ms, integer division is fine
		self.animator=Animator(fps_in,animation.next)

	def do_mkmovie_snapshots(self,samplerate,anim_style):
		import tempfile
		tmp=os.path.join(tempfile.gettempdir(),'cvu')
		try:
			os.mkdir(tmp)
		except OSError: # directory already exists
			try:
				for file in os.listdir(os.path.join(tmp,'cvu')):
					os.unlink(os.path.join(tmp,'cvu',file))
			except (OSError,IOError) as e:
				self.error_dialog('Tried to remove files from /tmp/cvu and '
					'failed.  If you dont understand the error message '
					'that follows, contact the developer\n%s' % str(e))
				return
		except IOError as e: # no permissions
			self.error_dialog(str(e))
			return
		self.curdir=os.getcwd()
		os.chdir(tmp)
		self.do_mkmovie_animate(samplerate,anim_style,snapshot_folder=tmp)

	def do_mkmovie_x11grab(self):
		mmw=self.make_movie_window
		xs,ys=self.scene.scene_editor.control.GetScreenPositionTuple()
		ys+=32 #size of mayavi toolbar.  constant.  can't be removed.
		xe,ye=tuple(self.scene.scene_editor.get_size())
		cmd = ('ffmpeg -loglevel error -y -f x11grab -s %ix%i -r %i -b %i '
			'-i :0.0+%i,%i %s'
			% (xe,ye,mmw.framerate,mmw.bitrate*1024,xs,ys,mmw.savefile))
		try:
			self.ffmpeg_process=util.sh_cmd_retproc(cmd)
			self.do_mkmovie_animate(mmw.samplerate,mmw.anim_style)
		except util.CVUError as e:
			self.error_dialog(str(e))

	def do_mkmovie_finish(self):
		mmw=self.make_movie_window
		self.mk_movie_lbl='Make movie'
		if self.animator:
			self.animator.timer.Stop()
		del self.animator
		if self.curdir:
			os.chdir(self.curdir)
		del self.curdir
		if mmw.type=='x11grab':
		#x11grab method
			self.ffmpeg_process.communicate('q')
			if self.ffmpeg_process.returncode:
				self.error_dialog('ffmpeg failed with error code %s' %
					self.ffmpeg_process.returncode)
				return
			del self.ffmpeg_process
		else:
			import tempfile
			tmp=tempfile.gettempdir()
			regex=os.path.join(tmp,'cvu','movie%05d.png')
			cmd=('ffmpeg -loglevel error -y -f image2 -r %i -b %i -i %s -y '
				'-sameq %s -pass 2' % 
				(mmw.framerate,mmw.bitrate*1024,regex,mmw.savefile))
			try:
				util.sh_cmd(cmd)
			except util.CVUError as e:
				self.error_dialog(str(e))
				return
		print "Movie saved successfully to %s" % mmw.savefile

	## MISC GUI BUTTONS ##
	def _options_button_fired(self):
		self.opts.edit_traits()

	def _custom_colormap_button_fired(self):
		self.custom_colormap_window.edit_traits()

	def _color_legend_button_fired(self):
		self.color_legend_window.edit_traits()

	def _draw_stuff_button_fired(self):
		mlab.draw()
		self.circ_fig.canvas.draw()
		self.conn_mat.request_redraw()

	#removed long ago
	def _up_node_button_fired(self):
		if self.curr_node==None:
			return
		elif self.curr_node==self.nr_labels-1:	
			self.display_node(0)
		else:
			self.display_node(self.curr_node+1)

	#removed long ago
	def _down_node_button_fired(self):
		if self.curr_node==None:
			return
		elif self.curr_node==0:
			self.display_node(self.nr_labels-1)
		else:
			self.display_node(self.curr_node-1)
	
	def _center_adjmat_button_fired(self):
		for ax in [self.xa,self.ya]:
			ax.mapper.range.high_setting=self.nr_labels
			ax.mapper.range.low_setting=0

	def _about_button_fired(self):
		self.about_window.edit_traits()

def preproc():
	#load label names from specified text file for ordering
	labnam,ign=util.read_parcellation_textfile(args['parcorder'])
	adjlabs=args['adjorder']
	
	#load adjacency matrix.  entries in order of labnam
	adj = util.loadmat(args['adjmat'],args['field']) 

	#load surface for visual display
	surf_fname=os.path.join(args['subjdir'],args['subject'],'surf',
		'lh.%s'%args['surftype'])
	surf_struct=util.loadsurf(surf_fname,args['surftype'])

	#load parcellation and vertex positions
	labv=util.loadannot(args['parc'],args['subject'],args['subjdir'],
		surf_type=args['surftype'])

	#calculate label positions from vertex positions
	if args['subject']=='fsavg5':
		lab_pos,labv=util.calcparc(labv,labnam,quiet=quiet,
			parcname=args['parc'])
	else:
		lab_pos,labv=util.calcparc(labv,labnam,quiet=quiet,
			parcname=args['parc'],subjdir=args['subjdir'],
			subject=args['subject'],lhsurf=surf_struct[0],rhsurf=surf_struct[2])

	# Package dataloc and modality into tuple for passing
	datainfo =(args['dataloc'],args['modality'],args['partitiontype'],
		args['maxedges'],args['subject'],args['parc'],args['adjmat'])

	# Return tuple with summary required data
	return lab_pos,adj,labnam,adjlabs,surf_struct,labv,datainfo

if __name__ == "__main__":
	#gc.set_debug(gc.DEBUG_LEAK)
	cvu_args = preproc()
	cvu = Cvu(cvu_args)
	cvu.configure_traits()



#    (C) Roan LaPlante 2013 rlaplant@nmr.mgh.harvard.edu
#
#	 This file is part of cvu, the Connectome Visualization Utility.
#
#    cvu is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from matplotlib.colors import LinearSegmentedColormap
from mayavi.core import lut_manager
from pylab import get_cmap
from traits.api import HasTraits, Enum, Bool, File, Range, Str

class CustomColormap(HasTraits):
	lut_list=lut_manager.lut_mode_list()
	lut_list.remove('black-white')
	lut_list.remove('blue-red')
	lut_list.append('custom_heat')

	imgs_path = 'cmap_images'

	map_type = Enum('default','scalar','activation','connmat')
	cmap = Enum(lut_list)
	reverse = Bool
	fname = File
	label = Str
	threshold = Range(0.0,0.5,0.2)

	def __init__(self,type):
		self.map_type=type

	def _cmap_default(self):
		if self.map_type=='scalar': return 'BuGn'
		elif self.map_type=='activation': return 'YlOrRd'
		elif self.map_type=='connmat': return 'RdYlBu'
		else: return 'cool' #return default

	def _reverse_default(self):
		if self.map_type=='connmat': return True
		else: return False

	def _label_default(self):
		if self.map_type=='scalar': return 'Scalars Colormap'
		elif self.map_type=='activation': return 'Conns Colormap'
		elif self.map_type=='connmat': return 'Matrix Colormap'
		else: return 'Default Colormap'

def get_cmap_pl(map):
	'''From a CustomColormap instance, return a LinearSegmentedColormap
describing that CustomColormap'''
	if map.cmap=='file' and map.fname:
		return LinearSegmentedColormap.from_list('file',
			lut_manager.parse_lut_file(map.fname))
	elif map.cmap=='custom_heat':
		return gen_heatmap(t=map.threshold,reversed=map.reverse)
	elif map.reverse:
		return get_cmap(map.cmap+'_r')
	else:
		return get_cmap(map.cmap)

def gen_heatmap(t=.2,two_tailed=True,reversed=False):
	'''Generates a heatmap that has red-to-yellow at the top, blue-to-light-blue
at the bottom, and gray in the middle.  If the map is one-tailed, only the red
portion is displayed:

gen_heatmap(t=.2,two_tailed=True)
t -- the threshold used to start both tails of the heatmap.  This should be a
value between 0 and .5.  You are responsible for calculating the right value
from your statistics.

two-tailed -- if false, only the upper (red) part of the heatmap is shown'''

	#a very fancy algorithm to do things correctly when the map is inverted.
	#sorry about the readability
	def swapif(trip):
		if reversed: return ( 1-trip[0], trip[2], trip[1] )
		else: return trip

	def revif(tups):
		if reversed: l=list(tups); l.reverse(); return tuple(l)
		else: return tups

	if two_tailed and not reversed:
		cdict = {'red': revif(( swapif((0.0,0.0,0.6)),
							  	swapif((t  ,0.2,0.4)),
							  	swapif((1-t,0.4,1.0)),
							  	swapif((1.0,1.0,0.0)))),
				'green':revif(( swapif((0.0,0.0,1.0)),
								swapif((t  ,0.5,0.4)),
								swapif((1-t,0.4,0.0)),
								swapif((1.0,1.0,0.0)))),
				'blue': revif(( swapif((0.0,0.0,1.0)),
								swapif((t  ,1.0,0.4)),
								swapif((1-t,0.4,0.0)),
								swapif((1.0,0.5,0.0)))),}
	else:
		cdict = {'red': revif((	swapif((0.0,0.0,0.4)),
								swapif((1-t,0.4,1.0)),
								swapif((1.0,1.0,0.0)))),
				'green':revif(( swapif((0.0,0.0,0.4)),
								swapif((1-t,0.4,0.0)),
								swapif((1.0,1.0,0.0)))),
				'blue': revif(( swapif((0.0,0.0,0.4)),
								swapif((1-t,0.4,0.0)),
								swapif((1.0,0.5,0.0)))),}

	return LinearSegmentedColormap('heatmap_%.3f'%t,cdict,256)

def map_to_table(cmap,nvals=256):
	'''Takes a LinearSegmentedColormap, and returns a table of RGBA values
spanning that colormap.

	arguments:
	cmap - the LinearSegmentedColormap instance
	nvals- the number of values to span over.  The default is 256.'''

	return cmap(xrange(nvals),bytes=True)

def set_lut(mayavi_obj,map,use_vector_lut=False):
	'''Takes a mayavi object and a CustomColormap and sets the color LUT on thatobject to use the colormap referred to by the CustomColormap, handling all
edge cases

	arguments:
	mayavi_obj - the mayavi object.  This object MUST have a module_manager
		trait, which not all mayavi objects do.
	map - the CustomColormap instance
	use_vector_lut - if True, uses the vector_lut_manager instead of the
		scalar_lut_manager on the mayavi object.  Defaults to false.'''

	if use_vector_lut:
		lut_mgr=mayavi_obj.module_manager.vector_lut_manager
	else:
		lut_mgr=mayavi_obj.module_manager.scalar_lut_manager

	lut_mgr.file_name=map.fname	
	lut_mgr.reverse_lut=map.reverse

	if map.cmap=='custom_heat':
		lut_mgr.lut_mode='black-white' #this mode is not used and will always
									#trigger notifications when changed
		hmap = gen_heatmap(t=map.threshold)
		lut_mgr.lut.table = map_to_table(hmap)
	else:
		lut_mgr.lut_mode=map.cmap

